@@ -14,7 +14,7 @@ package MCE::Core::Input::Iterator;
 use strict;
 use warnings;
 
-our $VERSION = '1.810';
+our $VERSION = '1.833';
 
 ## Items below are folded into MCE.
 
@@ -46,14 +46,22 @@ sub _worker_user_iterator {
    my $_DAU_W_SOCK  = $self->{_dat_w_sock}->[$_chn];
    my $_lock_chn    = $self->{_lock_chn};
    my $_chunk_size  = $self->{chunk_size};
-   my $_I_FLG       = (!$/ || $/ ne $LF);
    my $_wuf         = $self->{_wuf};
 
-   my ($_dat_ex, $_dat_un);
+   my ($_dat_ex, $_dat_un, $_pid);
 
    if ($_lock_chn) {
-      $_dat_ex = sub {  sysread ( $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
-      $_dat_un = sub { syswrite ( $_DAT_LOCK->{_w_sock}, '0' ) };
+      $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
+
+      # inlined for performance
+      $_dat_ex = sub {
+         sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+            unless $_DAT_LOCK->{ $_pid };
+      };
+      $_dat_un = sub {
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            if $_DAT_LOCK->{ $_pid };
+      };
    }
 
    my ($_chunk_id, $_len, $_is_ref);
@@ -68,16 +76,17 @@ sub _worker_user_iterator {
    _WORKER_USER_ITERATOR__NEXT:
 
    while (1) {
-      undef $_ if (length > MAX_CHUNK_SIZE);
+      undef $_ if (length > MAX_GC_SIZE);
 
       $_ = '';
 
       ## Obtain the next chunk of data.
       {
-         local $\ = undef if (defined $\); local $/ = $LF if ($_I_FLG);
+         local $\ = undef if (defined $\);
+         local $/ = $LF   if ($/ ne $LF );
 
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_U_ITR . $LF . $_chn . $LF;
+         print {$_DAT_W_SOCK} OUTPUT_I_REF . $LF . $_chn . $LF;
          chomp($_len = <$_DAU_W_SOCK>);
 
          if ($_len < 0) {
