@@ -14,7 +14,7 @@ package MCE::Core::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '1.810';
+our $VERSION = '1.833';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -26,8 +26,13 @@ package # hide from rpm
 
 no warnings qw( threads recursion uninitialized );
 
-## POSIX.pm is a big module. The following constant covers most platforms.
-use constant { _WNOHANG => $^O eq 'solaris' ? 64 : 1 };
+## The POSIX module has many symbols. Try not loading it simply
+## to have WNOHANG. The following covers most platforms.
+
+use constant {
+   _WNOHANG => ( $INC{'POSIX.pm'} )
+      ? &POSIX::WNOHANG : ( $^O eq 'solaris' ) ? 64 : 1
+};
 
 use bytes;
 
@@ -85,40 +90,37 @@ sub _output_loop {
       $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
       $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
       $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT,
-      $_win32_ipc
+      @_delay_wid, $_size_completed, $_win32_ipc
    );
 
    ## -------------------------------------------------------------------------
    ## Callback return.
 
-   my $_cb_ret_a = sub {                          # CBK return array
+   my $_cb_reply = sub {
+      local $\ = $_O_SEP if ($_O_FLG);
+      local $/ = $_I_SEP if ($_I_FLG);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
+      no strict 'refs';
 
-      print {$_DAU_R_SOCK} $_len.$LF, $_buf;
+      if ( $_wa == WANTS_UNDEF ) {
+         $_cb->(@_);
+         return;
+      }
+      elsif ( $_wa == WANTS_ARRAY ) {
+         my @_ret = $_cb->(@_);
+         my $_buf = $self->{freeze}(\@_ret);
 
-      return;
-   };
+         return print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
+      }
 
-   my $_cb_ret_r = sub {                          # CBK return reference
+      my $_ret = $_cb->(@_);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
+      return print {$_DAU_R_SOCK} length($_ret).'0'.$LF, $_ret
+         if ( !ref $_ret && defined $_ret && !looks_like_number $_ret );
 
-      print {$_DAU_R_SOCK} WANTS_REF.$LF . $_len.$LF, $_buf;
+      my $_buf = $self->{freeze}([ $_ret ]);
 
-      return;
-   };
-
-   my $_cb_ret_s = sub {                          # CBK return scalar
-
-      $_len = (defined $_[0]) ? length $_[0] : -1;
-      local $\ = undef if (defined $\);
-
-      print {$_DAU_R_SOCK} WANTS_SCALAR.$LF . $_len.$LF, $_[0];
-
-      return;
+      return print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
    };
 
    ## -------------------------------------------------------------------------
@@ -145,7 +147,9 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+               }
                undef $_syn_flag;
             }
          }
@@ -175,7 +179,9 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+               }
                undef $_syn_flag;
             }
          }
@@ -262,7 +268,7 @@ sub _output_loop {
 
       ## ----------------------------------------------------------------------
 
-      OUTPUT_A_ARY.$LF => sub {                   # Array << Array
+      OUTPUT_A_REF.$LF => sub {                   # Input << Array ref
          my $_buf;
 
          if ($_offset_pos >= $_input_size || $_aborted) {
@@ -295,7 +301,7 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_S_GLB.$LF => sub {                   # Scalar << Glob FH
+      OUTPUT_G_REF.$LF => sub {                   # Input << Glob ref
          my $_buf = '';
 
          ## The logic below honors ('Ctrl/Z' in Windows, 'Ctrl/D' in Unix)
@@ -352,7 +358,37 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_U_ITR.$LF => sub {                   # User << Iterator
+      OUTPUT_H_REF.$LF => sub {                   # Input << Hash ref
+         my @_pairs;
+
+         if ($_offset_pos >= $_input_size || $_aborted) {
+            local $\ = undef if (defined $\);
+            print {$_DAU_R_SOCK} '0'.$LF;
+
+            return;
+         }
+
+         if ($_offset_pos + $_chunk_size - 1 < $_input_size) {
+            for my $_i ($_offset_pos .. $_offset_pos + $_chunk_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+         else {
+            for my $_i ($_offset_pos .. $_input_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+
+         my $_buf = $self->{freeze}(\@_pairs);
+
+         $_len = length $_buf; local $\ = undef if (defined $\);
+         print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
+         $_offset_pos += $_chunk_size;
+
+         return;
+      },
+
+      OUTPUT_I_REF.$LF => sub {                   # Input << Iter ref
          my $_buf;
 
          if ($_aborted) {
@@ -365,7 +401,7 @@ sub _output_loop {
          my @_ret_a = $_input_data->($_chunk_size);
 
          if (scalar @_ret_a > 1 || ref $_ret_a[0]) {
-            $_buf = $self->{freeze}( [ @_ret_a ] );
+            $_buf = $self->{freeze}([ @_ret_a ]);
             $_len = length $_buf; local $\ = undef if (defined $\);
             print {$_DAU_R_SOCK} $_len.'1'.$LF . (++$_chunk_id).$LF, $_buf;
 
@@ -388,92 +424,40 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_CBK.$LF => sub {                   # Callback w/ multiple args
-         my ($_buf, $_data_ref);
-
          chomp($_wa  = <$_DAU_R_SOCK>),
          chomp($_cb  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
-         $_data_ref = $self->{thaw}($_buf); undef $_buf;
+         read $_DAU_R_SOCK, my($_buf), $_len;
+         my $_data_ref = $self->{thaw}($_buf); undef $_buf;
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->(@{ $_data_ref });
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->(@{ $_data_ref });
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->(@{ $_data_ref });
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->(@{ $_data_ref });
       },
 
       OUTPUT_S_CBK.$LF => sub {                   # Callback w/ 1 scalar arg
-         my $_buf;
-
          chomp($_wa  = <$_DAU_R_SOCK>),
          chomp($_cb  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->($_buf);
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->($_buf);
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->($_buf);
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->($_buf);
       },
 
       OUTPUT_N_CBK.$LF => sub {                   # Callback w/ no args
-
          chomp($_wa = <$_DAU_R_SOCK>),
          chomp($_cb = <$_DAU_R_SOCK>);
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->();
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->();
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->();
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->();
       },
 
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_GTR.$LF => sub {                   # Gather array/ref
-         my $_buf;
-
          chomp($_task_id = <$_DAU_R_SOCK>),
          chomp($_len     = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if ($_is_c_ref[$_task_id]) {
             local $_ = $self->{thaw}($_buf);
@@ -524,10 +508,8 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_O_SND.$LF => sub {                   # Send >> STDOUT
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if (defined $_user_output) {
             $_user_output->($_buf);
@@ -539,10 +521,8 @@ sub _output_loop {
       },
 
       OUTPUT_E_SND.$LF => sub {                   # Send >> STDERR
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if (defined $_user_error) {
             $_user_error->($_buf);
@@ -569,7 +549,9 @@ sub _output_loop {
 
             ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_file}), $| = 1)[0]);
+               local $!;
+               # IO::Handle->autoflush not available in older Perl.
+               select(( select($_sendto_fhs{$_file}), $| = 1 )[0]);
             }
          }
 
@@ -588,7 +570,7 @@ sub _output_loop {
          read $_DAU_R_SOCK, $_buf, $_len;
 
          unless (exists $_sendto_fhs{$_fd}) {
-            require IO::Handle unless (defined $IO::Handle::VERSION);
+            require IO::Handle unless $INC{'IO/Handle.pm'};
 
             $_sendto_fhs{$_fd} = IO::Handle->new();
             $_sendto_fhs{$_fd}->fdopen($_fd, 'w')
@@ -598,7 +580,9 @@ sub _output_loop {
 
             ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_fd}), $| = 1)[0]);
+               local $!;
+               # IO::Handle->autoflush not available in older Perl.
+               select(( select($_sendto_fhs{$_fd}), $| = 1 )[0]);
             }
          }
 
@@ -611,7 +595,6 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_B_SYN.$LF => sub {                   # Barrier sync - begin
-
          if (!defined $_sync_cnt || $_sync_cnt == 0) {
             $_syn_flag = 1, $_sync_cnt = 0;
          }
@@ -621,7 +604,9 @@ sub _output_loop {
             : $self->{_total_running};
 
          if (++$_sync_cnt == $_total_running) {
-            for (1 .. $_total_running) { syswrite $_BSB_W_SOCK, $LF }
+            for my $_i (1 .. $_total_running) {
+               1 until syswrite($_BSB_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+            }
             undef $_syn_flag;
          }
 
@@ -629,38 +614,102 @@ sub _output_loop {
       },
 
       OUTPUT_E_SYN.$LF => sub {                   # Barrier sync - end
-
          if (--$_sync_cnt == 0) {
             my $_total_running = ($_has_user_tasks)
                ? $self->{_task}->[0]->{_total_running}
                : $self->{_total_running};
 
-            for (1 .. $_total_running) { syswrite $_BSE_W_SOCK, $LF }
+            for my $_i (1 .. $_total_running) {
+               1 until syswrite($_BSE_W_SOCK, $LF) || ($! && !$!{'EINTR'});
+            }
          }
 
          return;
       },
 
       OUTPUT_S_IPC.$LF => sub {                   # Change to win32 IPC
-
-         syswrite $_DAT_R_SOCK, $LF;
+         1 until syswrite($_DAT_R_SOCK, $LF) || ($! && !$!{'EINTR'});
 
          $_win32_ipc = 1, goto _LOOP unless $_win32_ipc;
 
          return;
       },
 
+      OUTPUT_P_NFY.$LF => sub {                   # Progress notification
+         chomp($_len = <$_DAU_R_SOCK>);
+
+         $self->{progress}->( $_size_completed += $_len );
+
+         return;
+      },
+
+      OUTPUT_S_DIR.$LF => sub {                   # Make/get sess_dir
+         print {$_DAU_R_SOCK} $self->sess_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_T_DIR.$LF => sub {                   # Make/get tmp_dir
+         print {$_DAU_R_SOCK} $self->tmp_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_I_DLY.$LF => sub {                   # Interval delay
+         my $_tasks = $_has_user_tasks ? $self->{user_tasks} : undef;
+
+         chomp($_task_id = <$_DAU_R_SOCK>);
+
+         my $_interval = ($_tasks && exists $_tasks->[$_task_id]{interval})
+            ? $_tasks->[$_task_id]{interval}
+            : $self->{interval};
+
+         if ( $_interval ) {
+            my $_max_workers = ($_tasks)
+               ? $_tasks->[$_task_id]{max_workers}
+               : $self->{max_workers};
+
+            $_delay_wid[$_task_id] = 1
+               if (++$_delay_wid[$_task_id] > $_max_workers);
+
+            my $_nodes  = $_interval->{max_nodes};
+            my $_id     = $_interval->{node_id};
+            my $_delay  = $_interval->{delay} * $_nodes;
+
+            my $_app_tb = $_delay * $_max_workers;
+            my $_app_st = $_interval->{_time} + ($_delay / $_nodes * $_id);
+            my $_wrk_st = ($_delay_wid[$_task_id] - 1) * $_delay + $_app_st;
+
+            $_delay = $_wrk_st - time;
+
+            if ($_delay < 0.0 && $_app_tb) {
+               my $_count = int($_delay * -1 / $_app_tb + 0.5) + 1;
+               $_delay += $_app_tb * $_count;
+               $_interval->{_time} = time if ($_count > 2e9);
+            }
+
+            ($_delay > 0.0)
+               ? print {$_DAU_R_SOCK} $_delay.$LF
+               : print {$_DAU_R_SOCK} '0'.$LF;
+         }
+         else {
+            print {$_DAU_R_SOCK} '0'.$LF;
+         }
+
+         return;
+      },
+
    );
 
    ## -------------------------------------------------------------------------
 
    local ($!, $?, $_);
 
+   $_aborted = $_chunk_id = $_eof_flag = $_size_completed = 0;
    $_has_user_tasks = (defined $self->{user_tasks}) ? 1 : 0;
    $_cs_one_flag = ($self->{chunk_size} == 1) ? 1 : 0;
-   $_aborted = $_chunk_id = $_eof_flag = 0;
 
-   $_max_retries  = $self->{max_retries};
+   $_max_retries  = $self->{max_retries} || 0;
    $_on_post_exit = $self->{on_post_exit};
    $_on_post_run  = $self->{on_post_run};
    $_chunk_size   = $self->{chunk_size};
@@ -673,10 +722,10 @@ sub _output_loop {
    if ($_max_retries && !$_on_post_exit) {
       $_on_post_exit = sub {
          my ($self, $_e, $_retry_cnt) = @_;
-         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: Chunk $_e->{id} failed");
+         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: chunk $_e->{id} failed");
 
          ($_retry_cnt < $_max_retries)
-            ? print {*STDERR} "$_msg, retrying chunk attempt #${_cnt}\n"
+            ? print {*STDERR} "$_msg, retrying chunk attempt # ${_cnt}\n"
             : print {*STDERR} "$_msg\n";
 
          $self->restart_worker;
@@ -715,7 +764,12 @@ sub _output_loop {
    if (defined $_input_data && ref $_input_data eq 'ARRAY') {
       $_input_size = @{ $_input_data };
       $_offset_pos = 0;
-   } else {
+   }
+   elsif (defined $_input_data && ref $_input_data eq 'HASH') {
+      $_input_size = scalar( keys %{ $_input_data } );
+      $_offset_pos = 0;
+   }
+   else {
       $_input_size = $_offset_pos = 0;
    }
 
@@ -741,16 +795,17 @@ sub _output_loop {
       binmode $_MCE_STDERR;
    }
 
+   ## Autoflush STDERR-STDOUT handles if requested.
    ## Make MCE_STDOUT the default handle.
-   ## Flush STDERR/STDOUT handles if requested.
 
    my $_old_hndl = select $_MCE_STDOUT;
 
-   if ($self->{flush_stdout}) {
-      local $|; select((select($_MCE_STDOUT), $| = 1)[0]);
-   }
-   if ($self->{flush_stderr}) {
-      local $|; select((select($_MCE_STDERR), $| = 1)[0]);
+   {
+      local $!;
+      # IO::Handle->autoflush not available in older Perl.
+      select($_MCE_STDERR), $| = 1 if ($self->{flush_stderr});
+      select($_MCE_STDOUT), $| = 1 if ($self->{flush_stdout});
+      select($_MCE_STDOUT);
    }
 
    ## -------------------------------------------------------------------------
@@ -827,10 +882,10 @@ sub _output_loop {
 
       my $_val_bytes = "\x00\x00\x00\x00";
       my $_ptr_bytes = unpack( 'I', pack('P', $_val_bytes) );
-      my ($_count, $_done, $_nbytes, $_start) = (1, 0);
+      my ($_done, $_count, $_nbytes, $_start) = (0);
 
       while (!$_done) {
-         $_start = time;
+         $_start = time, $_count = 1;
 
          # MSWin32 FIONREAD
          IOCTL: ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);
@@ -845,8 +900,6 @@ sub _output_loop {
             goto IOCTL;
          }
 
-         $_count = 1;
-
          do {
             sysread($_DAT_R_SOCK, $_func, 8);
             $_done = 1, last() unless length($_func) == 8;
