@@ -14,7 +14,14 @@ package MCE::Core::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '1.810';
+our $VERSION = '1.833';
+
+my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $_tid = $_has_threads ? threads->tid() : 0;
+
+sub CLONE {
+   $_tid = threads->tid() if $_has_threads;
+}
 
 ## Items below are folded into MCE.
 
@@ -49,8 +56,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_F_SND.$LF . $_chn.$LF;
-         print {$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] };
+         print({$_DAT_W_SOCK} OUTPUT_F_SND.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] });
          $_dat_un->() if $_lock_chn;
       }
 
@@ -64,8 +71,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_D_SND.$LF . $_chn.$LF;
-         print {$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] };
+         print({$_DAT_W_SOCK} OUTPUT_D_SND.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_val.$LF . length(${ $_[0] }).$LF, ${ $_[0] });
          $_dat_un->() if $_lock_chn;
       }
 
@@ -78,8 +85,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_O_SND.$LF . $_chn.$LF;
-         print {$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] };
+         print({$_DAT_W_SOCK} OUTPUT_O_SND.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] });
          $_dat_un->() if $_lock_chn;
       }
 
@@ -92,8 +99,8 @@ use bytes;
 
       if (length ${ $_[0] }) {
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} OUTPUT_E_SND.$LF . $_chn.$LF;
-         print {$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] };
+         print({$_DAT_W_SOCK} OUTPUT_E_SND.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} length(${ $_[0] }).$LF, ${ $_[0] });
          $_dat_un->() if $_lock_chn;
       }
 
@@ -114,69 +121,48 @@ use bytes;
          $_wa = WANTS_SCALAR;
       }
 
+      local $\ = undef if (defined $\);
+
       ## Crossover: Send arguments
 
-      if (scalar @{ $_aref } > 0) {               ## Multiple Args >> Callback
-         if (scalar @{ $_aref } > 1 || ref $_aref->[0]) {
-            $_tag = OUTPUT_A_CBK;
-            $_buf = $self->{freeze}($_aref);
-            $_len = length $_buf; local $\ = undef if (defined $\);
-
-            $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
-            print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_buf;
-
-         }
-         else {                                   ## Scalar >> Callback
-            $_tag = OUTPUT_S_CBK;
-            $_len = length $_aref->[0]; local $\ = undef if (defined $\);
-
-            $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
-            print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_aref->[0];
-         }
+      if ( ! @{ $_aref } ) {
+         $_dat_ex->() if $_lock_chn;
+         print({$_DAT_W_SOCK} OUTPUT_N_CBK.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_wa.$LF . $_val.$LF);
       }
-      else {                                      ## No Args >> Callback
-         $_tag = OUTPUT_N_CBK;
-         local $\ = undef if (defined $\);
+      elsif ( @{ $_aref } == 1 && !ref $_aref->[0] &&
+              defined $_aref->[0] && !looks_like_number($_aref->[0]) ) {
+         $_len = length $_aref->[0];
 
          $_dat_ex->() if $_lock_chn;
-         print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
-         print {$_DAU_W_SOCK} $_wa.$LF . $_val.$LF;
-      }
-
-      ## Crossover: Receive return value
-
-      if ($_wa == WANTS_UNDEF) {
-         $_dat_un->() if $_lock_chn;
-         return;
-      }
-      elsif ($_wa == WANTS_ARRAY) {
-         local $/ = $LF if (!$/ || $/ ne $LF);
-         chomp($_len = <$_DAU_W_SOCK>);
-
-         read($_DAU_W_SOCK, $_buf, $_len || 0);
-         $_dat_un->() if $_lock_chn;
-
-         return @{ $self->{thaw}($_buf) };
+         print({$_DAT_W_SOCK} OUTPUT_S_CBK.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_aref->[0]);
       }
       else {
-         local $/ = $LF if (!$/ || $/ ne $LF);
-         chomp($_wa = <$_DAU_W_SOCK>);
-         chomp($_len     = <$_DAU_W_SOCK>);
+         $_buf = $self->{freeze}($_aref);
+         $_len = length $_buf;
 
-         if ($_len >= 0) {
-            read($_DAU_W_SOCK, $_buf, $_len || 0);
-            $_dat_un->() if $_lock_chn;
-
-            return $_buf if ($_wa == WANTS_SCALAR);
-            return $self->{thaw}($_buf);
-         }
-         else {
-            $_dat_un->() if $_lock_chn;
-            return;
-         }
+         $_dat_ex->() if $_lock_chn;
+         print({$_DAT_W_SOCK} OUTPUT_A_CBK.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_wa.$LF . $_val.$LF . $_len.$LF, $_buf);
       }
+
+      ## Crossover: Receive value
+
+      if ( $_wa ) {
+         local $/ = $LF if ($/ ne $LF);
+         chomp(my $_len = <$_DAU_W_SOCK>);
+
+         my $_frozen = chop($_len);
+         read $_DAU_W_SOCK, my($_buf), $_len;
+         $_dat_un->() if $_lock_chn;
+
+         return ( $_wa != WANTS_ARRAY )
+            ? $_frozen ? ($self->{thaw}($_buf))->[0] : $_buf
+            : @{ $self->{thaw}($_buf) };
+      }
+
+      $_dat_un->() if $_lock_chn;
    }
 
    ## -------------------------------------------------------------------------
@@ -187,7 +173,8 @@ use bytes;
 
       return unless (scalar @{ $_aref });
 
-      if (scalar @{ $_aref } > 1 || ref $_aref->[0]) {
+      if (scalar @{ $_aref } > 1 || ref $_aref->[0] ||
+            !defined $_aref->[0] || looks_like_number $_aref->[0]) {
          $_tag = OUTPUT_A_GTR;
          $_buf = $self->{freeze}($_aref);
          $_len = length $_buf;
@@ -198,8 +185,8 @@ use bytes;
             $_len = length $_aref->[0]; local $\ = undef if (defined $\);
 
             $_dat_ex->() if $_lock_chn;
-            print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
-            print {$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_aref->[0];
+            print({$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF),
+            print({$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_aref->[0]);
             $_dat_un->() if $_lock_chn;
 
             return;
@@ -213,8 +200,8 @@ use bytes;
       local $\ = undef if (defined $\);
 
       $_dat_ex->() if $_lock_chn;
-      print {$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF;
-      print {$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_buf;
+      print({$_DAT_W_SOCK} $_tag.$LF . $_chn.$LF),
+      print({$_DAU_W_SOCK} $_task_id.$LF . $_len.$LF, $_buf);
       $_dat_un->() if $_lock_chn;
 
       return;
@@ -270,7 +257,8 @@ use bytes;
          }
       }
       else {
-         my $_fh = qualify_to_ref($_glob, caller);
+         require Symbol unless $INC{'Symbol.pm'};
+         my $_fh = Symbol::qualify_to_ref($_glob, caller);
          local $\ = undef if (defined $\);
          print {$_fh} ${ $_data_ref };
       }
@@ -292,12 +280,22 @@ use bytes;
       $_task_id    = $self->{_task_id};
 
       if ($_lock_chn) {
-         $_dat_ex = sub {  sysread ( $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
-         $_dat_un = sub { syswrite ( $_DAT_LOCK->{_w_sock}, '0' ) };
+         # inlined for performance
+         $_dat_ex = sub {
+            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+            sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+               unless $_DAT_LOCK->{ $_pid };
+         };
+         $_dat_un = sub {
+            my $_pid = $_has_threads ? $$ .'.'. $_tid : $$;
+            syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+               if $_DAT_LOCK->{ $_pid };
+         };
       }
 
       {
-         local ($|, $!);
+         local $!;
+         # IO::Handle->autoflush not available in older Perl.
          select(( select(*STDERR), $| = 1 )[0]) if defined(fileno *STDERR);
          select(( select(*STDOUT), $| = 1 )[0]) if defined(fileno *STDOUT);
       }
@@ -321,6 +319,29 @@ use bytes;
    sub _do_user_func {
 
       my ($self, $_chunk, $_chunk_id) = @_;
+      my $_size = 0;
+
+      if ($self->{progress} && $self->{_task_id} == 0) {
+         # use_slurpio
+         if (ref $_chunk eq 'SCALAR') {
+            $_size += length ${ $_chunk };
+         }
+         # sequence and bounds_only
+         elsif ($self->{sequence} && $self->{bounds_only}) {
+            my $_seq = $self->{sequence};
+            my $_step = (ref $_seq eq 'ARRAY') ? $_seq->[2] : $_seq->{step};
+            $_size += int(abs($_chunk->[0] - $_chunk->[1]) / abs($_step)) + 1;
+         }
+         # workers clear {input_data} to conserve memory when array ref
+         # otherwise, /path/to/infile or scalar reference
+         elsif ($self->{input_data}) {
+            map { $_size += length } @{ $_chunk };
+         }
+         # array or sequence
+         else {
+            $_size += (ref $_chunk eq 'ARRAY') ? @{ $_chunk } : 1;
+         }
+      }
 
       $self->{_retry} = [ $_chunk, $_chunk_id, $self->{max_retries} ]
          if ($self->{max_retries});
@@ -328,6 +349,15 @@ use bytes;
       $self->{_chunk_id} = $_chunk_id;
       $_user_func->($self, $_chunk, $_chunk_id);
 
+      if ($self->{progress} && $self->{_task_id} == 0) {
+         local $\ = undef if (defined $\);
+
+         $_dat_ex->() if $_lock_chn;
+         print({$_DAT_W_SOCK} OUTPUT_P_NFY.$LF . $_chn.$LF),
+         print({$_DAU_W_SOCK} $_size.$LF);
+         $_dat_un->() if $_lock_chn;
+      }
+
       return;
    }
 
@@ -359,6 +389,7 @@ sub _worker_do {
    $self->{_single_dim} = $_params_ref->{_single_dim};
    $self->{use_slurpio} = $_params_ref->{_use_slurpio};
    $self->{parallel_io} = $_params_ref->{_parallel_io};
+   $self->{progress}    = $_params_ref->{_progress};
    $self->{max_retries} = $_params_ref->{_max_retries};
    $self->{RS}          = $_params_ref->{_RS};
 
@@ -375,7 +406,7 @@ sub _worker_do {
    my $_task_name  = $self->{task_name};
 
    ## Do not override params if defined in user_tasks during instantiation.
-   for my $_p (qw(bounds_only chunk_size interval sequence user_args)) {
+   for my $_p (qw(bounds_only chunk_size sequence user_args)) {
       if (defined $_params_ref->{"_${_p}"}) {
          $self->{$_p} = $_params_ref->{"_${_p}"}
             unless (defined $self->{_task}->{$_p});
@@ -385,25 +416,11 @@ sub _worker_do {
    ## Assign user function.
    $self->{_wuf} = \&_do_user_func;
 
-   ## Set time_block & start_time values for interval.
-   if (defined $self->{interval}) {
-      my $_i     = $self->{interval};
-      my $_delay = $_i->{delay} * $_i->{max_nodes};
-
-      $self->{_i_app_tb} = $_delay * $self->{max_workers};
-
-      $self->{_i_app_st} =
-         $_i->{_time} + ($_delay / $_i->{max_nodes} * $_i->{node_id});
-
-      $self->{_i_wrk_st} =
-         ($self->{_task_wid} - 1) * $_delay + $self->{_i_app_st};
-   }
-
    ## Call user_begin if defined.
    if (defined $self->{user_begin}) {
       $self->{_chunk_id} = 0;
       $self->{user_begin}($self, $_task_id, $_task_name);
-      $self->sync if ($_task_id == 0 && defined $self->{init_relay});
+      $self->sync() if ($_task_id == 0 && defined $self->{init_relay});
    }
 
    ## Retry chunk if previous attempt died.
@@ -416,37 +433,42 @@ sub _worker_do {
    ## Call worker function.
    if ($_run_mode eq 'sequence') {
       require MCE::Core::Input::Sequence
-         unless (defined $MCE::Core::Input::Sequence::VERSION);
+         unless $INC{'MCE/Core/Input/Sequence.pm'};
       _worker_sequence_queue($self);
    }
    elsif (defined $self->{_task}->{sequence}) {
       require MCE::Core::Input::Generator
-         unless (defined $MCE::Core::Input::Generator::VERSION);
+         unless $INC{'MCE/Core/Input/Generator.pm'};
       _worker_sequence_generator($self);
    }
    elsif ($_run_mode eq 'array') {
       require MCE::Core::Input::Request
-         unless (defined $MCE::Core::Input::Request::VERSION);
+         unless $INC{'MCE/Core/Input/Request.pm'};
       _worker_request_chunk($self, REQUEST_ARRAY);
    }
    elsif ($_run_mode eq 'glob') {
       require MCE::Core::Input::Request
-         unless (defined $MCE::Core::Input::Request::VERSION);
+         unless $INC{'MCE/Core/Input/Request.pm'};
       _worker_request_chunk($self, REQUEST_GLOB);
    }
+   elsif ($_run_mode eq 'hash') {
+      require MCE::Core::Input::Request
+         unless $INC{'MCE/Core/Input/Request.pm'};
+      _worker_request_chunk($self, REQUEST_HASH);
+   }
    elsif ($_run_mode eq 'iterator') {
       require MCE::Core::Input::Iterator
-         unless (defined $MCE::Core::Input::Iterator::VERSION);
+         unless $INC{'MCE/Core/Input/Iterator.pm'};
       _worker_user_iterator($self);
    }
    elsif ($_run_mode eq 'file') {
       require MCE::Core::Input::Handle
-         unless (defined $MCE::Core::Input::Handle::VERSION);
+         unless $INC{'MCE/Core/Input/Handle.pm'};
       _worker_read_handle($self, READ_FILE, $_params_ref->{_input_file});
    }
    elsif ($_run_mode eq 'memory') {
       require MCE::Core::Input::Handle
-         unless (defined $MCE::Core::Input::Handle::VERSION);
+         unless $INC{'MCE/Core/Input/Handle.pm'};
       _worker_read_handle($self, READ_MEMORY, $self->{input_data});
    }
    elsif (defined $self->{user_func}) {
@@ -461,10 +483,12 @@ sub _worker_do {
    ## Call user_end if defined.
    if (defined $self->{user_end}) {
       $self->{_chunk_id} = 0;
-      $self->sync if ($_task_id == 0 && defined $self->{init_relay});
+      $self->sync() if ($_task_id == 0 && defined $self->{init_relay});
       $self->{user_end}($self, $_task_id, $_task_name);
    }
 
+   delete $self->{_wuf};
+
    ## Check nested Hobo workers not yet joined.
    MCE::Hobo->finish('MCE') if $INC{'MCE/Hobo.pm'};
 
@@ -473,8 +497,8 @@ sub _worker_do {
 
    $_DAT_LOCK->lock() if $_lock_chn;
 
-   print {$_DAT_W_SOCK} OUTPUT_W_DNE.$LF . $_chn.$LF;
-   print {$_DAU_W_SOCK} $_task_id.$LF;
+   print({$_DAT_W_SOCK} OUTPUT_W_DNE.$LF . $_chn.$LF),
+   print({$_DAU_W_SOCK} $_task_id.$LF);
 
    $_DAT_LOCK->unlock() if $_lock_chn;
 
@@ -504,9 +528,6 @@ sub _worker_loop {
    my $_job_delay  = $self->{job_delay};
    my $_wid        = $self->{_wid};
 
-   my $_com_ex = sub {  sysread ( $_COM_LOCK->{_r_sock}, my $_b, 1 ) };
-   my $_com_un = sub { syswrite ( $_COM_LOCK->{_w_sock}, '0' ) };
-
    if ( $^O eq 'MSWin32' ) {
       lock $MCE::_WIN_LOCK;
    }
@@ -515,7 +536,7 @@ sub _worker_loop {
 
       {
          local $\ = undef; local $/ = $LF;
-         $_com_ex->();
+         $_COM_LOCK->lock();
 
          ## Wait for the next job request.
          $_response = <$_COM_W_SOCK>;
@@ -523,17 +544,17 @@ sub _worker_loop {
 
          ## Return if instructed to exit.
          if ($_response eq "_exit\n") {
-            $_com_un->();
+            $_COM_LOCK->unlock();
             return;
          }
 
          ## Process send request.
          if ($_response eq "_data\n") {
-            chomp($_len = <$_COM_W_SOCK>);
-            read $_COM_W_SOCK, $_buf, $_len;
+            chomp($_len = <$_COM_W_SOCK>),
+            read($_COM_W_SOCK, $_buf, $_len);
 
-            print {$_COM_W_SOCK} $_wid.$LF;
-            $_com_un->();
+            print {$_COM_W_SOCK} $LF;
+            $_COM_LOCK->unlock();
 
             $self->{user_data} = $self->{thaw}($_buf);
             undef $_buf;
@@ -545,11 +566,11 @@ sub _worker_loop {
 
          ## Process normal request.
          elsif ($_response =~ /\d+/) {
-            chomp($_len = <$_COM_W_SOCK>);
-            read $_COM_W_SOCK, $_buf, $_len;
+            chomp($_len = <$_COM_W_SOCK>),
+            read($_COM_W_SOCK, $_buf, $_len);
 
-            print {$_COM_W_SOCK} $_wid.$LF;
-            $_com_un->();
+            print {$_COM_W_SOCK} $LF;
+            $_COM_LOCK->unlock();
 
             $_params_ref = $self->{thaw}($_buf);
             undef $_buf;
@@ -565,7 +586,7 @@ sub _worker_loop {
       _worker_do($self, {}), next if ($_response eq "_data\n");
 
       ## Wait here until MCE completes job submission to all workers.
-      sysread $self->{_bse_r_sock}, my($_b), 1;
+      1 until sysread($self->{_bse_r_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
 
       ## Normal request.
       if (defined $_job_delay && $_job_delay > 0.0) {
@@ -578,7 +599,7 @@ sub _worker_loop {
    ## Notify the main process a worker has ended. The following is executed
    ## when an invalid reply was received above (not likely to occur).
 
-   $_com_un->();
+   $_COM_LOCK->unlock();
 
    die "Worker ($self->{_wid}) has ended prematurely";
 }
@@ -610,16 +631,19 @@ sub _worker_main {
    my $_use_threads = (defined $_task->{use_threads})
       ? $_task->{use_threads} : $self->{use_threads};
 
-   if ($INC{'threads.pm'} && $_use_threads) {
-      $self->{_exit_pid} = 'TID_' . threads->tid();
+   if ($_has_threads && $_use_threads) {
+      $self->{_exit_pid} = 'TID_' . $_tid;
    } else {
       $self->{_exit_pid} = 'PID_' . $$;
    }
 
+   my $_running_inside_eval = $^S;
+
    local $SIG{__DIE__} = sub {
       if (!defined $^S || $^S) {
-         if ( ($INC{'threads.pm'} && threads->tid() != 0) ||
-               $ENV{'PERL_IPERL_RUNNING'}
+         if ( ($_has_threads && $_tid != 0) ||
+               $ENV{'PERL_IPERL_RUNNING'} ||
+               $_running_inside_eval
          ) {
             # thread env or running inside IPerl, check stack trace
             my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
@@ -646,7 +670,6 @@ sub _worker_main {
    $self->{max_workers} = $_task->{max_workers} if ($_task->{max_workers});
    $self->{chunk_size}  = $_task->{chunk_size}  if ($_task->{chunk_size});
    $self->{gather}      = $_task->{gather}      if ($_task->{gather});
-   $self->{interval}    = $_task->{interval}    if ($_task->{interval});
    $self->{sequence}    = $_task->{sequence}    if ($_task->{sequence});
    $self->{bounds_only} = $_task->{bounds_only} if ($_task->{bounds_only});
    $self->{task_name}   = $_task->{task_name}   if ($_task->{task_name});
@@ -656,8 +679,6 @@ sub _worker_main {
    $self->{user_end}    = $_task->{user_end}    if ($_task->{user_end});
 
    ## Init runtime vars. Obtain handle to lock files.
-   my $_mce_sid  = $self->{_mce_sid};
-   my $_sess_dir = $self->{_sess_dir};
    my $_chn;
 
    if (defined $_params && exists $_params->{_chn}) {
@@ -678,7 +699,8 @@ sub _worker_main {
    ) };
 
    ## Call MCE::Shared's init routine if present; enables parallel IPC.
-   MCE::Shared::init($_wid) if ($INC{'MCE/Shared.pm'});
+   ## For threads, init is called automatically via the CLONE feature.
+   MCE::Shared::init($_wid) if (!$_use_threads && $INC{'MCE/Shared.pm'});
 
    _do_send_init($self);
 
