@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.810';
+our $VERSION = '1.833';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
@@ -64,7 +64,7 @@ sub import {
       $_p->{FREEZE}      = shift, next if ( $_arg eq 'freeze' );
       $_p->{THAW}        = shift, next if ( $_arg eq 'thaw' );
 
-      ## Sereal 3.008+, if available, is used automatically by MCE 1.800.
+      ## Sereal 3.015+, if available, is used automatically by MCE 1.8+.
       if ( $_arg eq 'sereal' ) {
          if ( shift eq '0' ) {
             require Storable;
@@ -200,22 +200,22 @@ sub run_seq (@) {
    }
 
    for my $_i ($_start_pos .. @_ - 1) {
-      my $_ref = ref $_[$_i];
+      my $_r = ref $_[$_i];
 
-      if ($_ref eq '' || $_ref eq 'HASH' || $_ref eq 'ARRAY') {
+      if ($_r eq '' || $_r =~ /^Math::/ || $_r eq 'HASH' || $_r eq 'ARRAY') {
          $_pos = $_i;
 
-         if ($_ref eq '') {
+         if ($_r eq '' || $_r =~ /^Math::/) {
             $_begin = $_[$_pos]; $_end = $_[$_pos + 1];
             $_params->{$_pid}{sequence} = [
                $_[$_pos], $_[$_pos + 1], $_[$_pos + 2], $_[$_pos + 3]
             ];
          }
-         elsif ($_ref eq 'HASH') {
+         elsif ($_r eq 'HASH') {
             $_begin = $_[$_pos]->{begin}; $_end = $_[$_pos]->{end};
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
-         elsif ($_ref eq 'ARRAY') {
+         elsif ($_r eq 'ARRAY') {
             $_begin = $_[$_pos]->[0]; $_end = $_[$_pos]->[1];
             $_params->{$_pid}{sequence} = $_[$_pos];
          }
@@ -231,7 +231,7 @@ sub run_seq (@) {
    _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
-   $_params->{$_pid}{sequence_run} = 1;
+   $_params->{$_pid}{sequence_run} = undef;
 
    if (defined $_pos) {
       pop @_ for ($_pos .. @_ - 1);
@@ -311,8 +311,8 @@ sub run (@) {
    my $_input_data; my $_max_workers = $_def->{$_pkg}{MAX_WORKERS};
    my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'SCALAR' || $_r =~ /^(?:GLOB|FileHandle|IO::)/) {
-      $_input_data = shift if (@_ == 1);
+   if (@_ == 1 && $_r =~ /^(?:ARRAY|HASH|SCALAR|GLOB|FileHandle|IO::)/) {
+      $_input_data = shift;
    }
 
    if (defined (my $_p = $_params->{$_pid})) {
@@ -343,7 +343,7 @@ sub run (@) {
 
    ## -------------------------------------------------------------------------
 
-   MCE::_save_state();
+   MCE::_save_state($_MCE->{$_pid});
 
    if ($_init_mce) {
       $_MCE->{$_pid}->shutdown() if (defined $_MCE->{$_pid});
@@ -370,7 +370,7 @@ sub run (@) {
             next if ($_ eq 'input_data');
             next if ($_ eq 'sequence_run');
 
-            _croak("MCE::Flow: ($_) is not a valid constructor argument")
+            _croak("$_tag: ($_) is not a valid constructor argument")
                unless (exists $MCE::_valid_fields_new{$_});
 
             $_opts{$_} = $_p->{$_};
@@ -495,7 +495,7 @@ MCE::Flow - Parallel flow model for building creative applications
 
 =head1 VERSION
 
-This document describes MCE::Flow version 1.810
+This document describes MCE::Flow version 1.833
 
 =head1 DESCRIPTION
 
@@ -504,64 +504,64 @@ This module was created to help one harness user_tasks within MCE.
 
 It is trivial to parallelize with mce_stream shown below.
 
-   ## Native map function
-   my @a = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..10000;
+ ## Native map function
+ my @a = map { $_ * 4 } map { $_ * 3 } map { $_ * 2 } 1..10000;
 
-   ## Same as with MCE::Stream (processing from right to left)
-   @a = mce_stream
-        sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
+ ## Same as with MCE::Stream (processing from right to left)
+ @a = mce_stream
+      sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
 
-   ## Pass an array reference to have writes occur simultaneously
-   mce_stream \@a,
-        sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
+ ## Pass an array reference to have writes occur simultaneously
+ mce_stream \@a,
+      sub { $_ * 4 }, sub { $_ * 3 }, sub { $_ * 2 }, 1..10000;
 
 However, let's have MCE::Flow compute the same in parallel. MCE::Queue
 will be used for data flow among the sub-tasks.
 
-   use MCE::Flow;
-   use MCE::Queue;
+ use MCE::Flow;
+ use MCE::Queue;
 
 This calls for preserving output order.
 
-   sub preserve_order {
-      my %tmp; my $order_id = 1; my $gather_ref = $_[0];
-      @{ $gather_ref } = ();  ## clear the array (optional)
+ sub preserve_order {
+    my %tmp; my $order_id = 1; my $gather_ref = $_[0];
+    @{ $gather_ref } = ();  ## clear the array (optional)
 
-      return sub {
-         my ($data_ref, $chunk_id) = @_;
-         $tmp{$chunk_id} = $data_ref;
+    return sub {
+       my ($data_ref, $chunk_id) = @_;
+       $tmp{$chunk_id} = $data_ref;
 
-         while (1) {
-            last unless exists $tmp{$order_id};
-            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
-         }
+       while (1) {
+          last unless exists $tmp{$order_id};
+          push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
+       }
 
-         return;
-      };
-   }
+       return;
+    };
+ }
 
 Two queues are needed for data flow between the 3 sub-tasks. Notice task_end
 and how the value from $task_name is used for determining which task has ended.
 
-   my $b = MCE::Queue->new;
-   my $c = MCE::Queue->new;
+ my $b = MCE::Queue->new;
+ my $c = MCE::Queue->new;
 
-   sub task_end {
-      my ($mce, $task_id, $task_name) = @_;
+ sub task_end {
+    my ($mce, $task_id, $task_name) = @_;
 
-      if (defined $mce->{user_tasks}->[$task_id + 1]) {
-         my $n_workers = $mce->{user_tasks}->[$task_id + 1]->{max_workers};
+    if (defined $mce->{user_tasks}->[$task_id + 1]) {
+       my $n_workers = $mce->{user_tasks}->[$task_id + 1]->{max_workers};
 
-         if ($task_name eq 'a') {
-            $b->enqueue((undef) x $n_workers);
-         }
-         elsif ($task_name eq 'b') {
-            $c->enqueue((undef) x $n_workers);
-         }
-      }
+       if ($task_name eq 'a') {
+          $b->enqueue((undef) x $n_workers);
+       }
+       elsif ($task_name eq 'b') {
+          $c->enqueue((undef) x $n_workers);
+       }
+    }
 
-      return;
-   }
+    return;
+ }
 
 Next are the 3 sub-tasks. The first one reads input and begins the flow.
 The 2nd task dequeues, performs the calculation, and enqueues into the next.
@@ -571,58 +571,58 @@ Although serialization is done for you automatically, it is done here to save
 from double serialization. This is the fastest approach for passing data
 between sub-tasks. Thus, the least overhead.
 
-   sub task_a {
-      my @ans; my ($mce, $chunk_ref, $chunk_id) = @_;
+ sub task_a {
+    my @ans; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      push @ans, map { $_ * 2 } @{ $chunk_ref };
-      $b->enqueue(MCE->freeze([ \@ans, $chunk_id ]));
+    push @ans, map { $_ * 2 } @{ $chunk_ref };
+    $b->enqueue(MCE->freeze([ \@ans, $chunk_id ]));
 
-      return;
-   }
+    return;
+ }
 
-   sub task_b {
-      my ($mce) = @_;
+ sub task_b {
+    my ($mce) = @_;
 
-      while (1) {
-         my @ans; my $chunk = $b->dequeue;
-         last unless defined $chunk;
+    while (1) {
+       my @ans; my $chunk = $b->dequeue;
+       last unless defined $chunk;
 
-         $chunk = MCE->thaw($chunk);
-         push @ans, map { $_ * 3 } @{ $chunk->[0] };
-         $c->enqueue(MCE->freeze([ \@ans, $chunk->[1] ]));
-      }
+       $chunk = MCE->thaw($chunk);
+       push @ans, map { $_ * 3 } @{ $chunk->[0] };
+       $c->enqueue(MCE->freeze([ \@ans, $chunk->[1] ]));
+    }
 
-      return;
-   }
+    return;
+ }
 
-   sub task_c {
-      my ($mce) = @_;
+ sub task_c {
+    my ($mce) = @_;
 
-      while (1) {
-         my @ans; my $chunk = $c->dequeue;
-         last unless defined $chunk;
+    while (1) {
+       my @ans; my $chunk = $c->dequeue;
+       last unless defined $chunk;
 
-         $chunk = MCE->thaw($chunk);
-         push @ans, map { $_ * 4 } @{ $chunk->[0] };
-         MCE->gather(\@ans, $chunk->[1]);
-      }
+       $chunk = MCE->thaw($chunk);
+       push @ans, map { $_ * 4 } @{ $chunk->[0] };
+       MCE->gather(\@ans, $chunk->[1]);
+    }
 
-      return;
-   }
+    return;
+ }
 
 In summary, MCE::Flow builds out a MCE instance behind the scene and starts
 running. The task_name (shown), max_workers, and use_threads options can take
 an anonymous array for specifying the values uniquely per each sub-task.
 
-   my @a;
+ my @a;
 
-   mce_flow {
-      task_name => [ 'a', 'b', 'c' ], task_end => \&task_end,
-      gather => preserve_order(\@a)
+ mce_flow {
+    task_name => [ 'a', 'b', 'c' ], task_end => \&task_end,
+    gather => preserve_order(\@a)
 
-   }, \&task_a, \&task_b, \&task_c, 1..10000;
+ }, \&task_a, \&task_b, \&task_c, 1..10000;
 
-   print "@a\n";
+ print "@a\n";
 
 If speed is not a concern and wanting to rid of all the MCE->freeze and
 MCE->thaw statements, simply enqueue and dequeue 2 items at a time.
@@ -631,69 +631,69 @@ Or better yet, see L<MCE::Step> introduced in MCE 1.506.
 First, task_end must be updated. The number of undef(s) must match the number
 of workers times the dequeue count. Otherwise, the script will stall.
 
-   sub task_end {
-      ...
-         if ($task_name eq 'a') {
-          # $b->enqueue((undef) x $n_workers);
-            $b->enqueue((undef) x ($n_workers * 2));
-         }
-         elsif ($task_name eq 'b') {
-          # $c->enqueue((undef) x $n_workers);
-            $c->enqueue((undef) x ($n_workers * 2));
-         }
-      ...
-   }
+ sub task_end {
+    ...
+       if ($task_name eq 'a') {
+        # $b->enqueue((undef) x $n_workers);
+          $b->enqueue((undef) x ($n_workers * 2));
+       }
+       elsif ($task_name eq 'b') {
+        # $c->enqueue((undef) x $n_workers);
+          $c->enqueue((undef) x ($n_workers * 2));
+       }
+    ...
+ }
 
 Next, the 3 sub-tasks enqueuing and dequeuing 2 elements at a time.
 
-   sub task_a {
-      my @ans; my ($mce, $chunk_ref, $chunk_id) = @_;
+ sub task_a {
+    my @ans; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      push @ans, map { $_ * 2 } @{ $chunk_ref };
-      $b->enqueue(\@ans, $chunk_id);
+    push @ans, map { $_ * 2 } @{ $chunk_ref };
+    $b->enqueue(\@ans, $chunk_id);
 
-      return;
-   }
+    return;
+ }
 
-   sub task_b {
-      my ($mce) = @_;
+ sub task_b {
+    my ($mce) = @_;
 
-      while (1) {
-         my @ans; my ($chunk_ref, $chunk_id) = $b->dequeue(2);
-         last unless defined $chunk_ref;
+    while (1) {
+       my @ans; my ($chunk_ref, $chunk_id) = $b->dequeue(2);
+       last unless defined $chunk_ref;
 
-         push @ans, map { $_ * 3 } @{ $chunk_ref };
-         $c->enqueue(\@ans, $chunk_id);
-      }
+       push @ans, map { $_ * 3 } @{ $chunk_ref };
+       $c->enqueue(\@ans, $chunk_id);
+    }
 
-      return;
-   }
+    return;
+ }
 
-   sub task_c {
-      my ($mce) = @_;
+ sub task_c {
+    my ($mce) = @_;
 
-      while (1) {
-         my @ans; my ($chunk_ref, $chunk_id) = $c->dequeue(2);
-         last unless defined $chunk_ref;
+    while (1) {
+       my @ans; my ($chunk_ref, $chunk_id) = $c->dequeue(2);
+       last unless defined $chunk_ref;
 
-         push @ans, map { $_ * 4 } @{ $chunk_ref };
-         MCE->gather(\@ans, $chunk_id);
-      }
+       push @ans, map { $_ * 4 } @{ $chunk_ref };
+       MCE->gather(\@ans, $chunk_id);
+    }
 
-      return;
-   }
+    return;
+ }
 
 Finally, run as usual.
 
-   my @a;
+ my @a;
 
-   mce_flow {
-      task_name => [ 'a', 'b', 'c' ], task_end => \&task_end,
-      gather => preserve_order(\@a)
+ mce_flow {
+    task_name => [ 'a', 'b', 'c' ], task_end => \&task_end,
+    gather => preserve_order(\@a)
 
-   }, \&task_a, \&task_b, \&task_c, 1..10000;
+ }, \&task_a, \&task_b, \&task_c, 1..10000;
 
-   print "@a\n";
+ print "@a\n";
 
 =head1 SYNOPSIS when CHUNK_SIZE EQUALS 1
 
@@ -708,70 +708,70 @@ variable $_ when chunk_size equals 1. Otherwise, $_ points to $chunk_ref
 containing many items. Basically, line 2 below may be omitted from your code
 when using $_. One can call MCE->chunk_id to obtain the current chunk id.
 
-   line 1:  user_func => sub {
-   line 2:     my ($mce, $chunk_ref, $chunk_id) = @_;
-   line 3:
-   line 4:     $_ points to $chunk_ref->[0]
-   line 5:        in MCE 1.5 when chunk_size == 1
-   line 6:
-   line 7:     $_ points to $chunk_ref
-   line 8:        in MCE 1.5 when chunk_size  > 1
-   line 9:  }
+ line 1:  user_func => sub {
+ line 2:     my ($mce, $chunk_ref, $chunk_id) = @_;
+ line 3:
+ line 4:     $_ points to $chunk_ref->[0]
+ line 5:        in MCE 1.5 when chunk_size == 1
+ line 6:
+ line 7:     $_ points to $chunk_ref
+ line 8:        in MCE 1.5 when chunk_size  > 1
+ line 9:  }
 
 Follow this synopsis when chunk_size equals one. Looping is not required from
 inside the first block. Hence, the block is called once per each item.
 
-   ## Exports mce_flow, mce_flow_f, and mce_flow_s
-   use MCE::Flow;
+ ## Exports mce_flow, mce_flow_f, and mce_flow_s
+ use MCE::Flow;
 
-   MCE::Flow::init {
-      chunk_size => 1
-   };
+ MCE::Flow::init {
+    chunk_size => 1
+ };
 
-   ## Array or array_ref
-   mce_flow sub { do_work($_) }, 1..10000;
-   mce_flow sub { do_work($_) }, [ 1..10000 ];
+ ## Array or array_ref
+ mce_flow sub { do_work($_) }, 1..10000;
+ mce_flow sub { do_work($_) }, [ 1..10000 ];
 
-   ## File_path, glob_ref, or scalar_ref
-   mce_flow_f sub { chomp; do_work($_) }, "/path/to/file";
-   mce_flow_f sub { chomp; do_work($_) }, $file_handle;
-   mce_flow_f sub { chomp; do_work($_) }, \$scalar;
+ ## File_path, glob_ref, or scalar_ref
+ mce_flow_f sub { chomp; do_work($_) }, "/path/to/file";
+ mce_flow_f sub { chomp; do_work($_) }, $file_handle;
+ mce_flow_f sub { chomp; do_work($_) }, \$scalar;
 
-   ## Sequence of numbers (begin, end [, step, format])
-   mce_flow_s sub { do_work($_) }, 1, 10000, 5;
-   mce_flow_s sub { do_work($_) }, [ 1, 10000, 5 ];
+ ## Sequence of numbers (begin, end [, step, format])
+ mce_flow_s sub { do_work($_) }, 1, 10000, 5;
+ mce_flow_s sub { do_work($_) }, [ 1, 10000, 5 ];
 
-   mce_flow_s sub { do_work($_) }, {
-      begin => 1, end => 10000, step => 5, format => undef
-   };
+ mce_flow_s sub { do_work($_) }, {
+    begin => 1, end => 10000, step => 5, format => undef
+ };
 
 =head1 SYNOPSIS when CHUNK_SIZE is GREATER THAN 1
 
 Follow this synopsis when chunk_size equals 'auto' or greater than 1.
 This means having to loop through the chunk from inside the first block.
 
-   use MCE::Flow;
+ use MCE::Flow;
 
-   MCE::Flow::init {          ## Chunk_size defaults to 'auto' when
-      chunk_size => 'auto'    ## not specified. Therefore, the init
-   };                         ## function may be omitted.
+ MCE::Flow::init {          ## Chunk_size defaults to 'auto' when
+    chunk_size => 'auto'    ## not specified. Therefore, the init
+ };                         ## function may be omitted.
 
-   ## Syntax is shown for mce_flow for demonstration purposes.
-   ## Looping inside the block is the same for mce_flow_f and
-   ## mce_flow_s.
+ ## Syntax is shown for mce_flow for demonstration purposes.
+ ## Looping inside the block is the same for mce_flow_f and
+ ## mce_flow_s.
 
-   mce_flow sub { do_work($_) for (@{ $_ }) }, 1..10000;
+ mce_flow sub { do_work($_) for (@{ $_ }) }, 1..10000;
 
-   ## Same as above, resembles code using the Core API.
+ ## Same as above, resembles code using the Core API.
 
-   mce_flow sub {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
+ mce_flow sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      for (@{ $chunk_ref }) {
-         do_work($_);
-      }
+    for (@{ $chunk_ref }) {
+       do_work($_);
+    }
 
-   }, 1..10000;
+ }, 1..10000;
 
 Chunking reduces the number of IPC calls behind the scene. Think in terms of
 chunks whenever processing a large amount of data. For relatively small data,
@@ -781,22 +781,22 @@ choosing 1 for chunk_size is fine.
 
 The following list options which may be overridden when loading the module.
 
-   use Sereal qw( encode_sereal decode_sereal );
-   use CBOR::XS qw( encode_cbor decode_cbor );
-   use JSON::XS qw( encode_json decode_json );
+ use Sereal qw( encode_sereal decode_sereal );
+ use CBOR::XS qw( encode_cbor decode_cbor );
+ use JSON::XS qw( encode_json decode_json );
 
-   use MCE::Flow
-       max_workers => 8,                # Default 'auto'
-       chunk_size => 500,               # Default 'auto'
-       tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
-       freeze => \&encode_sereal,       # \&Storable::freeze
-       thaw => \&decode_sereal          # \&Storable::thaw
-   ;
+ use MCE::Flow
+     max_workers => 8,                # Default 'auto'
+     chunk_size => 500,               # Default 'auto'
+     tmp_dir => "/path/to/app/tmp",   # $MCE::Signal::tmp_dir
+     freeze => \&encode_sereal,       # \&Storable::freeze
+     thaw => \&decode_sereal          # \&Storable::thaw
+ ;
 
-From MCE 1.8 onwards, Sereal 3.008+ is loaded automatically if available.
+From MCE 1.8 onwards, Sereal 3.015+ is loaded automatically if available.
 Specify C<Sereal => 0> to use Storable instead.
 
-   use MCE::Flow Sereal => 0;
+ use MCE::Flow Sereal => 0;
 
 =head1 CUSTOMIZING MCE
 
@@ -810,43 +810,43 @@ The init function accepts a hash of MCE options. Unlike with MCE::Stream,
 both gather and bounds_only options may be specified when calling init
 (not shown below).
 
-   use MCE::Flow;
+ use MCE::Flow;
 
-   MCE::Flow::init {
-      chunk_size => 1, max_workers => 4,
+ MCE::Flow::init {
+    chunk_size => 1, max_workers => 4,
 
-      user_begin => sub {
-         print "## ", MCE->wid, " started\n";
-      },
+    user_begin => sub {
+       print "## ", MCE->wid, " started\n";
+    },
 
-      user_end => sub {
-         print "## ", MCE->wid, " completed\n";
-      }
-   };
+    user_end => sub {
+       print "## ", MCE->wid, " completed\n";
+    }
+ };
 
-   my %a = mce_flow sub { MCE->gather($_, $_ * $_) }, 1..100;
+ my %a = mce_flow sub { MCE->gather($_, $_ * $_) }, 1..100;
 
-   print "\n", "@a{1..100}", "\n";
+ print "\n", "@a{1..100}", "\n";
 
-   -- Output
+ -- Output
 
-   ## 3 started
-   ## 2 started
-   ## 4 started
-   ## 1 started
-   ## 2 completed
-   ## 4 completed
-   ## 3 completed
-   ## 1 completed
+ ## 3 started
+ ## 2 started
+ ## 4 started
+ ## 1 started
+ ## 2 completed
+ ## 4 completed
+ ## 3 completed
+ ## 1 completed
 
-   1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
-   400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
-   1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
-   2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
-   3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
-   5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
-   7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
-   10000
+ 1 4 9 16 25 36 49 64 81 100 121 144 169 196 225 256 289 324 361
+ 400 441 484 529 576 625 676 729 784 841 900 961 1024 1089 1156
+ 1225 1296 1369 1444 1521 1600 1681 1764 1849 1936 2025 2116 2209
+ 2304 2401 2500 2601 2704 2809 2916 3025 3136 3249 3364 3481 3600
+ 3721 3844 3969 4096 4225 4356 4489 4624 4761 4900 5041 5184 5329
+ 5476 5625 5776 5929 6084 6241 6400 6561 6724 6889 7056 7225 7396
+ 7569 7744 7921 8100 8281 8464 8649 8836 9025 9216 9409 9604 9801
+ 10000
 
 =back
 
@@ -858,75 +858,118 @@ each code block.
 Unlike MCE::Stream which processes from right-to-left, MCE::Flow begins
 with the first code block, thus processing from left-to-right.
 
-   use threads;
-   use MCE::Flow;
+ use threads;
+ use MCE::Flow;
 
-   my @a = mce_flow {
-      task_name   => [ 'a', 'b', 'c' ],
-      max_workers => [  3,   4,   2, ],
-      use_threads => [  1,   0,   0, ],
+ my @a = mce_flow {
+    task_name   => [ 'a', 'b', 'c' ],
+    max_workers => [  3,   4,   2, ],
+    use_threads => [  1,   0,   0, ],
 
-      user_end => sub {
-         my ($mce, $task_id, $task_name) = @_;
-         MCE->print("$task_id - $task_name completed\n");
-      },
+    user_end => sub {
+       my ($mce, $task_id, $task_name) = @_;
+       MCE->print("$task_id - $task_name completed\n");
+    },
 
-      task_end => sub {
-         my ($mce, $task_id, $task_name) = @_;
-         MCE->print("$task_id - $task_name ended\n");
-      }
-   },
-   sub { sleep 1; },   ## 3 workers, named a
-   sub { sleep 2; },   ## 4 workers, named b
-   sub { sleep 3; };   ## 2 workers, named c
+    task_end => sub {
+       my ($mce, $task_id, $task_name) = @_;
+       MCE->print("$task_id - $task_name ended\n");
+    }
+ },
+ sub { sleep 1; },   ## 3 workers, named a
+ sub { sleep 2; },   ## 4 workers, named b
+ sub { sleep 3; };   ## 2 workers, named c
 
-   -- Output
+ -- Output
 
-   0 - a completed
-   0 - a completed
-   0 - a completed
-   0 - a ended
-   1 - b completed
-   1 - b completed
-   1 - b completed
-   1 - b completed
-   1 - b ended
-   2 - c completed
-   2 - c completed
-   2 - c ended
+ 0 - a completed
+ 0 - a completed
+ 0 - a completed
+ 0 - a ended
+ 1 - b completed
+ 1 - b completed
+ 1 - b completed
+ 1 - b completed
+ 1 - b ended
+ 2 - c completed
+ 2 - c completed
+ 2 - c ended
 
 =head1 API DOCUMENTATION
 
 Although input data is optional for MCE::Flow, the following assumes chunk_size
-equals 1 in order to demonstrate all the possibilities of passing input data
-into the code block.
+equals 1 in order to demonstrate all the possibilities for providing input data.
 
 =over 3
 
-=item MCE::Flow->run ( { input_data => iterator }, sub { code } )
-
-=item mce_flow { input_data => iterator }, sub { code }
-
-An iterator reference can by specified for input_data. The only other way
-is to specify input_data via MCE::Flow::init. This prevents MCE::Flow from
-configuring the iterator reference as another user task which will not work.
-
-Iterators are described under "SYNTAX for INPUT_DATA" at L<MCE::Core>.
-
-   MCE::Flow::init {
-      input_data => iterator
-   };
-
-   mce_flow sub { $_ };
-
 =item MCE::Flow->run ( sub { code }, list )
 
 =item mce_flow sub { code }, list
 
-Input data can be defined using a list.
+Input data may be defined using a list, an array ref, or a hash ref.
 
-   mce_flow sub { $_ }, 1..1000;
-   mce_flow sub { $_ }, [ 1..1000 ];
+Unlike MCE::Loop, Map, and Grep which take a block as C<{ ... }>, Flow takes a
+C<sub { ... }> or a code reference. The other difference is that the comma is
+needed after the block.
+
+ # $_ contains the item when chunk_size => 1
+
+ mce_flow sub { $_ }, 1..1000;
+ mce_flow sub { $_ }, \@list;
+
+ # chunking, any chunk_size => 1 or higher
+
+ my %res = mce_flow sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my %ret;
+    for my $item (@{ $chunk_ref }) {
+       $ret{$item} = $item * 2;
+    }
+    MCE->gather(%ret);
+ },
+ \@list;
+
+ # input hash, current API available since 1.828
+
+ my %res = mce_flow sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my %ret;
+    for my $key (keys %{ $chunk_ref }) {
+       $ret{$key} = $chunk_ref->{$key} * 2;
+    }
+    MCE->gather(%ret);
+ },
+ \%hash;
+
+ # unlike MCE::Loop, MCE::Flow doesn't need input to run
+
+ mce_flow { max_workers => 4 }, sub {
+    MCE->say( MCE->wid );
+ };
+
+ # ... and can run multiple tasks
+
+ mce_flow {
+    max_workers => [  1,   3  ],
+    task_name   => [ 'p', 'c' ]
+ },
+ sub {
+    # 1 producer
+    MCE->say( "producer: ", MCE->wid );
+ },
+ sub {
+    # 3 consumers
+    MCE->say( "consumer: ", MCE->wid );
+ };
+
+ # here, options are specified via init
+
+ MCE::Flow::init {
+    max_workers => [  1,   3  ],
+    task_name   => [ 'p', 'c' ]
+ };
+
+ mce_flow \&producer, \&consumers;
 
 =item MCE::Flow->run_file ( sub { code }, file )
 
@@ -935,26 +978,55 @@ Input data can be defined using a list.
 The fastest of these is the /path/to/file. Workers communicate the next offset
 position among themselves with zero interaction by the manager process.
 
-   mce_flow_f sub { $_ }, "/path/to/file";
-   mce_flow_f sub { $_ }, $file_handle;
-   mce_flow_f sub { $_ }, \$scalar;
+ # $_ contains the line when chunk_size => 1
+
+ mce_flow_f sub { $_ }, "/path/to/file";  # faster
+ mce_flow_f sub { $_ }, $file_handle;
+ mce_flow_f sub { $_ }, \$scalar;
+
+ # chunking, any chunk_size => 1 or higher
+
+ my %res = mce_flow_f sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my $buf = '';
+    for my $line (@{ $chunk_ref }) {
+       $buf .= $line;
+    }
+    MCE->gather($chunk_id, $buf);
+ },
+ "/path/to/file";
 
 =item MCE::Flow->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
 
 =item mce_flow_s sub { code }, $beg, $end [, $step, $fmt ]
 
-Sequence can be defined as a list, an array reference, or a hash reference.
+Sequence may be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
 optional. The format is passed to sprintf (% may be omitted below).
 
-   my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
+ my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
 
-   mce_flow_s sub { $_ }, $beg, $end, $step, $fmt;
-   mce_flow_s sub { $_ }, [ $beg, $end, $step, $fmt ];
+ # $_ contains the sequence number when chunk_size => 1
 
-   mce_flow_s sub { $_ }, {
-      begin => $beg, end => $end, step => $step, format => $fmt
-   };
+ mce_flow_s sub { $_ }, $beg, $end, $step, $fmt;
+ mce_flow_s sub { $_ }, [ $beg, $end, $step, $fmt ];
+
+ mce_flow_s sub { $_ }, {
+    begin => $beg, end => $end,
+    step => $step, format => $fmt
+ };
+
+ # chunking, any chunk_size => 1 or higher
+
+ my %res = mce_flow_s sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
+    my $buf = '';
+    for my $seq (@{ $chunk_ref }) {
+       $buf .= "$seq\n";
+    }
+    MCE->gather($chunk_id, $buf);
+ },
+ [ $beg, $end ];
 
 The sequence engine can compute 'begin' and 'end' items only, for the chunk,
 and not the items in between (hence boundaries only). This option applies
@@ -967,46 +1039,60 @@ for looping inside the block; e.g. Monte Carlo simulations.
 
 Time was measured using 1 worker to emphasize the difference.
 
-   use MCE::Flow;
+ use MCE::Flow;
 
-   MCE::Flow::init {
-      max_workers => 1, chunk_size => 1_250_000,
-      bounds_only => 1
-   };
+ MCE::Flow::init {
+    max_workers => 1, chunk_size => 1_250_000,
+    bounds_only => 1
+ };
 
-   ## For sequence, the input scalar $_ points to $chunk_ref
-   ## when chunk_size > 1, otherwise $chunk_ref->[0].
-   ##
-   ## mce_flow_s sub {
-   ##    my $begin = $_->[0]; my $end = $_->[-1];
-   ##
-   ##    for ($begin .. $end) {
-   ##       ...
-   ##    }
-   ##
-   ## }, 1, 10_000_000;
+ # Typically, the input scalar $_ contains the sequence number
+ # when chunk_size => 1, unless the bounds_only option is set
+ # which is the case here. Thus, $_ points to $chunk_ref.
 
-   mce_flow_s sub {
-      my ($mce, $chunk_ref, $chunk_id) = @_;
-      ## $chunk_ref contains 2 items, not 1_250_000
+ mce_flow_s sub {
+    my ($mce, $chunk_ref, $chunk_id) = @_;
 
-      my $begin = $chunk_ref->[ 0];
-      my $end   = $chunk_ref->[-1];   ## or $chunk_ref->[1]
+    # $chunk_ref contains 2 items, not 1_250_000
+    # my ( $begin, $end ) = ( $_->[0], $_->[1] );
 
-      MCE->printf("%7d .. %8d\n", $begin, $end);
+    my $begin = $chunk_ref->[0];
+    my $end   = $chunk_ref->[1];
 
-   }, 1, 10_000_000;
+    # for my $seq ( $begin .. $end ) {
+    #    ...
+    # }
 
-   -- Output
+    MCE->printf("%7d .. %8d\n", $begin, $end);
+ },
+ [ 1, 10_000_000 ];
 
-         1 ..  1250000
-   1250001 ..  2500000
-   2500001 ..  3750000
-   3750001 ..  5000000
-   5000001 ..  6250000
-   6250001 ..  7500000
-   7500001 ..  8750000
-   8750001 .. 10000000
+ -- Output
+
+       1 ..  1250000
+ 1250001 ..  2500000
+ 2500001 ..  3750000
+ 3750001 ..  5000000
+ 5000001 ..  6250000
+ 6250001 ..  7500000
+ 7500001 ..  8750000
+ 8750001 .. 10000000
+
+=item MCE::Flow->run ( { input_data => iterator }, sub { code } )
+
+=item mce_flow { input_data => iterator }, sub { code }
+
+An iterator reference may be specified for input_data. The only other way
+is to specify input_data via MCE::Flow::init. This prevents MCE::Flow from
+configuring the iterator reference as another user task which will not work.
+
+Iterators are described under section "SYNTAX for INPUT_DATA" at L<MCE::Core>.
+
+ MCE::Flow::init {
+    input_data => iterator
+ };
+
+ mce_flow sub { $_ };
 
 =back
 
@@ -1015,143 +1101,143 @@ Time was measured using 1 worker to emphasize the difference.
 Unlike MCE::Map where gather and output order are done for you automatically,
 the gather method is used to have results sent back to the manager process.
 
-   use MCE::Flow chunk_size => 1;
+ use MCE::Flow chunk_size => 1;
 
-   ## Output order is not guaranteed.
-   my @a = mce_flow sub { MCE->gather($_ * 2) }, 1..100;
-   print "@a\n\n";
+ ## Output order is not guaranteed.
+ my @a1 = mce_flow sub { MCE->gather($_ * 2) }, 1..100;
+ print "@a1\n\n";
 
-   ## Outputs to a hash instead (key, value).
-   my %h1 = mce_flow sub { MCE->gather($_, $_ * 2) }, 1..100;
-   print "@h1{1..100}\n\n";
+ ## Outputs to a hash instead (key, value).
+ my %h1 = mce_flow sub { MCE->gather($_, $_ * 2) }, 1..100;
+ print "@h1{1..100}\n\n";
 
-   ## This does the same thing due to chunk_id starting at one.
-   my %h2 = mce_flow sub { MCE->gather(MCE->chunk_id, $_ * 2) }, 1..100;
-   print "@h2{1..100}\n\n";
+ ## This does the same thing due to chunk_id starting at one.
+ my %h2 = mce_flow sub { MCE->gather(MCE->chunk_id, $_ * 2) }, 1..100;
+ print "@h2{1..100}\n\n";
 
-The gather method can be called multiple times within the block unlike return
+The gather method may be called multiple times within the block unlike return
 which would leave the block. Therefore, think of gather as yielding results
 immediately to the manager process without actually leaving the block.
 
-   use MCE::Flow chunk_size => 1, max_workers => 3;
+ use MCE::Flow chunk_size => 1, max_workers => 3;
 
-   my @hosts = qw(
-      hosta hostb hostc hostd hoste
-   );
+ my @hosts = qw(
+    hosta hostb hostc hostd hoste
+ );
 
-   my %h3 = mce_flow sub {
-      my ($output, $error, $status); my $host = $_;
+ my %h3 = mce_flow sub {
+    my ($output, $error, $status); my $host = $_;
 
-      ## Do something with $host;
-      $output = "Worker ". MCE->wid .": Hello from $host";
+    ## Do something with $host;
+    $output = "Worker ". MCE->wid .": Hello from $host";
 
-      if (MCE->chunk_id % 3 == 0) {
-         ## Simulating an error condition
-         local $? = 1; $status = $?;
-         $error = "Error from $host"
-      }
-      else {
-         $status = 0;
-      }
+    if (MCE->chunk_id % 3 == 0) {
+       ## Simulating an error condition
+       local $? = 1; $status = $?;
+       $error = "Error from $host"
+    }
+    else {
+       $status = 0;
+    }
 
-      ## Ensure unique keys (key, value) when gathering to
-      ## a hash.
-      MCE->gather("$host.out", $output);
-      MCE->gather("$host.err", $error) if (defined $error);
-      MCE->gather("$host.sta", $status);
+    ## Ensure unique keys (key, value) when gathering to
+    ## a hash.
+    MCE->gather("$host.out", $output);
+    MCE->gather("$host.err", $error) if (defined $error);
+    MCE->gather("$host.sta", $status);
 
-   }, @hosts;
+ }, @hosts;
 
-   foreach my $host (@hosts) {
-      print $h3{"$host.out"}, "\n";
-      print $h3{"$host.err"}, "\n" if (exists $h3{"$host.err"});
-      print "Exit status: ", $h3{"$host.sta"}, "\n\n";
-   }
+ foreach my $host (@hosts) {
+    print $h3{"$host.out"}, "\n";
+    print $h3{"$host.err"}, "\n" if (exists $h3{"$host.err"});
+    print "Exit status: ", $h3{"$host.sta"}, "\n\n";
+ }
 
-   -- Output
+ -- Output
 
-   Worker 3: Hello from hosta
-   Exit status: 0
+ Worker 3: Hello from hosta
+ Exit status: 0
 
-   Worker 2: Hello from hostb
-   Exit status: 0
+ Worker 2: Hello from hostb
+ Exit status: 0
 
-   Worker 1: Hello from hostc
-   Error from hostc
-   Exit status: 1
+ Worker 1: Hello from hostc
+ Error from hostc
+ Exit status: 1
 
-   Worker 3: Hello from hostd
-   Exit status: 0
+ Worker 3: Hello from hostd
+ Exit status: 0
 
-   Worker 2: Hello from hoste
-   Exit status: 0
+ Worker 2: Hello from hoste
+ Exit status: 0
 
 The following uses an anonymous array containing 3 elements when gathering
 data. Serialization is automatic behind the scene.
 
-   my %h3 = mce_flow sub {
-      ...
+ my %h3 = mce_flow sub {
+    ...
 
-      MCE->gather($host, [$output, $error, $status]);
+    MCE->gather($host, [$output, $error, $status]);
 
-   }, @hosts;
+ }, @hosts;
 
-   foreach my $host (@hosts) {
-      print $h3{$host}->[0], "\n";
-      print $h3{$host}->[1], "\n" if (defined $h3{$host}->[1]);
-      print "Exit status: ", $h3{$host}->[2], "\n\n";
-   }
+ foreach my $host (@hosts) {
+    print $h3{$host}->[0], "\n";
+    print $h3{$host}->[1], "\n" if (defined $h3{$host}->[1]);
+    print "Exit status: ", $h3{$host}->[2], "\n\n";
+ }
 
 Although MCE::Map comes to mind, one may want additional control when
 gathering data such as retaining output order.
 
-   use MCE::Flow;
+ use MCE::Flow;
 
-   sub preserve_order {
-      my %tmp; my $order_id = 1; my $gather_ref = $_[0];
+ sub preserve_order {
+    my %tmp; my $order_id = 1; my $gather_ref = $_[0];
 
-      return sub {
-         $tmp{ (shift) } = \@_;
+    return sub {
+       $tmp{ (shift) } = \@_;
 
-         while (1) {
-            last unless exists $tmp{$order_id};
-            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
-         }
+       while (1) {
+          last unless exists $tmp{$order_id};
+          push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
+       }
 
-         return;
-      };
-   }
+       return;
+    };
+ }
 
-   ## Workers persist for the most part after running. Though, not always
-   ## the case and depends on Perl. Pass a reference to a subroutine if
-   ## workers must persist; e.g. mce_flow { ... }, \&foo, 1..100000.
+ ## Workers persist for the most part after running. Though, not always
+ ## the case and depends on Perl. Pass a reference to a subroutine if
+ ## workers must persist; e.g. mce_flow { ... }, \&foo, 1..100000.
 
-   MCE::Flow::init {
-      chunk_size => 'auto', max_workers => 'auto'
-   };
+ MCE::Flow::init {
+    chunk_size => 'auto', max_workers => 'auto'
+ };
 
-   for (1..2) {
-      my @m2;
+ for (1..2) {
+    my @m2;
 
-      mce_flow {
-         gather => preserve_order(\@m2)
-      },
-      sub {
-         my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
+    mce_flow {
+       gather => preserve_order(\@m2)
+    },
+    sub {
+       my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-         ## Compute the entire chunk data at once.
-         push @a, map { $_ * 2 } @{ $chunk_ref };
+       ## Compute the entire chunk data at once.
+       push @a, map { $_ * 2 } @{ $chunk_ref };
 
-         ## Afterwards, invoke the gather feature, which
-         ## will direct the data to the callback function.
-         MCE->gather(MCE->chunk_id, @a);
+       ## Afterwards, invoke the gather feature, which
+       ## will direct the data to the callback function.
+       MCE->gather(MCE->chunk_id, @a);
 
-      }, 1..100000;
+    }, 1..100000;
 
-      print scalar @m2, "\n";
-   }
+    print scalar @m2, "\n";
+ }
 
-   MCE::Flow::finish;
+ MCE::Flow::finish;
 
 All 6 models support 'auto' for chunk_size unlike the Core API. Think of the
 models as the basis for providing JIT for MCE. They create the instance, tune
@@ -1160,36 +1246,36 @@ max_workers, and tune chunk_size automatically regardless of the hardware.
 The following does the same thing using the Core API. Workers persist after
 running.
 
-   use MCE;
+ use MCE;
 
-   sub preserve_order {
-      ...
-   }
+ sub preserve_order {
+    ...
+ }
 
-   my $mce = MCE->new(
-      max_workers => 'auto', chunk_size => 8000,
+ my $mce = MCE->new(
+    max_workers => 'auto', chunk_size => 8000,
 
-      user_func => sub {
-         my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
+    user_func => sub {
+       my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
 
-         ## Compute the entire chunk data at once.
-         push @a, map { $_ * 2 } @{ $chunk_ref };
+       ## Compute the entire chunk data at once.
+       push @a, map { $_ * 2 } @{ $chunk_ref };
 
-         ## Afterwards, invoke the gather feature, which
-         ## will direct the data to the callback function.
-         MCE->gather(MCE->chunk_id, @a);
-      }
-   );
+       ## Afterwards, invoke the gather feature, which
+       ## will direct the data to the callback function.
+       MCE->gather(MCE->chunk_id, @a);
+    }
+ );
 
-   for (1..2) {
-      my @m2;
+ for (1..2) {
+    my @m2;
 
-      $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
+    $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
 
-      print scalar @m2, "\n";
-   }
+    print scalar @m2, "\n";
+ }
 
-   $mce->shutdown;
+ $mce->shutdown;
 
 =head1 MANUAL SHUTDOWN
 
@@ -1203,15 +1289,15 @@ Workers remain persistent as much as possible after running. Shutdown occurs
 automatically when the script terminates. Call finish when workers are no
 longer needed.
 
-   use MCE::Flow;
+ use MCE::Flow;
 
-   MCE::Flow::init {
-      chunk_size => 20, max_workers => 'auto'
-   };
+ MCE::Flow::init {
+    chunk_size => 20, max_workers => 'auto'
+ };
 
-   mce_flow sub { ... }, 1..100;
+ mce_flow sub { ... }, 1..100;
 
-   MCE::Flow::finish;
+ MCE::Flow::finish;
 
 =back
 
