@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Signal - Temporary directory creation/cleanup and signal handling.
+## Temporary directory creation/cleanup and signal handling.
 ##
 ###############################################################################
 
@@ -9,105 +9,88 @@ package MCE::Signal;
 use strict;
 use warnings;
 
-use Carp ();
-use File::Path ();
-use Time::HiRes qw( sleep time );
-use Fcntl qw( :flock O_RDONLY );
-use base qw( Exporter );
+no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.608';
+our $VERSION = '1.833';
+
+## no critic (BuiltinFunctions::ProhibitStringyEval)
 
 our ($display_die_with_localtime, $display_warn_with_localtime);
 our ($main_proc_id, $prog_name, $tmp_dir);
 
+tie $tmp_dir, 'MCE::Signal::_tmpdir';
+
+use Carp ();
+
 BEGIN {
    $main_proc_id =  $$;
    $prog_name    =  $0;
    $prog_name    =~ s{^.*[\\/]}{}g;
-   $prog_name    =  'perl' if ($prog_name eq '-e');
+   $prog_name    =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
+
+   return;
 }
 
+use base qw( Exporter );
+use Time::HiRes ();
+
 our @EXPORT_OK = qw( $tmp_dir sys_cmd stop_and_exit );
 our %EXPORT_TAGS = (
    all     => \@EXPORT_OK,
    tmp_dir => [ qw( $tmp_dir ) ]
 );
 
+sub _NOOP {}
+
+END {
+   MCE::Signal->stop_and_exit($?) if ($$ == $main_proc_id);
+}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Process import, export, & module arguments.
 ##
 ###############################################################################
 
-sub _croak { $\ = undef; goto &Carp::croak; }
-sub _usage { return _croak "MCE::Signal error: ($_[0]) is not a valid option"; }
-sub _flag  { return 1; }
+sub _croak { $\ = undef; goto &Carp::croak }
+sub _usage { _croak "MCE::Signal error: ($_[0]) is not a valid option" }
+sub _flag  { 1 }
 
 my $_is_MSWin32   = ($^O eq 'MSWin32') ? 1 : 0;
 my $_keep_tmp_dir = 0;
-my $_no_sigmsg    = 0;
+my $_use_dev_shm  = 0;
 my $_no_kill9     = 0;
-my $_loaded;
+my $_imported;
 
 sub import {
+   my $_class = shift;
+   return if $_imported++;
 
-   my $_class = shift; return if ($_loaded++);
-
-   my @_export_args = ();
-   my $_no_setpgrp  = 0;
-   my $_setpgrp     = 0;
-   my $_use_dev_shm = 0;
+   my ($_no_setpgrp, $_no_sigmsg, $_setpgrp, @_export_args) = (0, 0, 0);
 
    while (my $_arg = shift) {
-
+      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
       $_keep_tmp_dir = _flag() and next if ($_arg eq '-keep_tmp_dir');
-      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
+      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
       $_no_kill9     = _flag() and next if ($_arg eq '-no_kill9');
 
+      # deprecated options for backwards compatibility
       $_no_setpgrp   = _flag() and next if ($_arg eq '-no_setpgrp');
-      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
-
-      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
+      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
 
       _usage($_arg) if ($_arg =~ /^-/);
+
       push @_export_args, $_arg;
    }
 
    local $Exporter::ExportLevel = 1;
    Exporter::import($_class, @_export_args);
 
- # ## MCE no longer calls setpgrp by default as of MCE 1.405.
- # ## Sets the current process group for the current process.
- # setpgrp(0,0) if ($_no_setpgrp == 0 && $^O ne 'MSWin32');
-
    ## Sets the current process group for the current process.
-   setpgrp(0,0) if ($_setpgrp == 1 && $^O ne 'MSWin32');
+   setpgrp(0,0) if ($_setpgrp == 1 && !$_is_MSWin32);
 
-   my ($_tmp_dir_base, $_count); $_count = 0;
-
-   if (exists $ENV{TEMP} && -d $ENV{TEMP} && -w $ENV{TEMP}) {
-      if ($_is_MSWin32) {
-         $_tmp_dir_base = $ENV{TEMP} . '/mce';
-         mkdir $_tmp_dir_base unless (-d $_tmp_dir_base);
-      }
-      else {
-         $_tmp_dir_base = $ENV{TEMP};
-      }
-   }
-   else {
-      $_tmp_dir_base = ($_use_dev_shm && -d '/dev/shm' && -w '/dev/shm')
-         ? '/dev/shm' : '/tmp';
-   }
-
-   _croak("MCE::Signal::import: ($_tmp_dir_base) is not writeable")
-      unless (-w $_tmp_dir_base);
-
-   ## Remove tainted'ness from $tmp_dir.
-   ($tmp_dir) = "$_tmp_dir_base/$prog_name.$$.$_count" =~ /(.*)/;
-
-   while ( !(mkdir $tmp_dir, 0770) ) {
-      ($tmp_dir) = ("$_tmp_dir_base/$prog_name.$$.".(++$_count)) =~ /(.*)/;
-   }
+   ## Make tmp_dir if caller requested it.
+   _make_tmpdir() if (grep /tmp_dir/, @_export_args);
 
    return;
 }
@@ -118,43 +101,165 @@ sub import {
 ##
 ###############################################################################
 
-my ($_mce_sess_dir_ref, $_mce_spawned_ref);
+## Set traps to catch signals.
+if ( !$_is_MSWin32 ) {
+   $SIG{HUP}  = \&stop_and_exit;  # UNIX SIG  1
+   $SIG{INT}  = \&stop_and_exit;  # UNIX SIG  2
+   $SIG{PIPE} = \&stop_and_exit;  # UNIX SIG 13
+   $SIG{QUIT} = \&stop_and_exit;  # UNIX SIG  3
+   $SIG{TERM} = \&stop_and_exit;  # UNIX SIG 15
 
-sub _set_session_vars {
-   ($_mce_sess_dir_ref, $_mce_spawned_ref) = @_;
-   return;
+   ## MCE handles the reaping of its children.
+   $SIG{CHLD} = 'DEFAULT';
 }
 
-## Set traps to catch signals.
-$SIG{XCPU} = \&stop_and_exit if (exists $SIG{XCPU});   ## UNIX SIG 24
-$SIG{XFSZ} = \&stop_and_exit if (exists $SIG{XFSZ});   ## UNIX SIG 25
+my $_safe_clean = 0;
 
-$SIG{HUP}  = \&stop_and_exit;                          ## UNIX SIG  1
-$SIG{INT}  = \&stop_and_exit;                          ## UNIX SIG  2
-$SIG{PIPE} = \&stop_and_exit;                          ## UNIX SIG 13
-$SIG{QUIT} = \&stop_and_exit;                          ## UNIX SIG  3
-$SIG{TERM} = \&stop_and_exit;                          ## UNIX SIG 15
+sub _make_tmpdir {
+   my ($_count, $_tmp_base_dir) = (0);
 
-## For a more reliable MCE, $SIG{CHLD} is set to 'DEFAULT'. MCE handles
-## the reaping of its children, especially when running multiple MCEs
-## simultaneously.
-##
-$SIG{CHLD} = 'DEFAULT' unless ($_is_MSWin32);
+   return $tmp_dir if (defined $tmp_dir && -d $tmp_dir && -w _);
+
+   if ($ENV{TEMP} && -d $ENV{TEMP} && -w _) {
+      if ($^O =~ /mswin|mingw|msys|cygwin/i) {
+         $_tmp_base_dir = $ENV{TEMP} . '/Perl-MCE';
+         mkdir $_tmp_base_dir unless -d $_tmp_base_dir;
+      } else {
+         $_tmp_base_dir = $ENV{TEMP};
+      }
+   }
+   else {
+      $_tmp_base_dir = ($_use_dev_shm && -d '/dev/shm' && -w _)
+         ? '/dev/shm' : '/tmp';
+   }
+
+   _croak("Error: MCE::Signal: ($_tmp_base_dir) is not writeable")
+      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_base_dir);
+
+   ## Remove tainted'ness from $tmp_dir.
+   ($tmp_dir) = "$_tmp_base_dir/$prog_name.$$.$_count" =~ /(.*)/;
+
+   while ( !(mkdir $tmp_dir, 0770) ) {
+      ($tmp_dir) = ("$_tmp_base_dir/$prog_name.$$.".(++$_count)) =~ /(.*)/;
+   }
+
+   $_safe_clean = 1;
+
+   return $tmp_dir;
+}
+
+sub _remove_tmpdir {
+   return if (!defined $tmp_dir || $tmp_dir eq '' || ! -d $tmp_dir);
+
+   if ($_keep_tmp_dir == 1) {
+      print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
+   }
+   elsif ($_safe_clean) {
+      if ($ENV{'TEMP'} && $^O =~ /mswin|mingw|msys|cygwin/i) {
+         ## remove tainted'ness
+         my ($_dir) = $ENV{'TEMP'} =~ /(.*)/;
+         chdir $_dir if -d $_dir;
+      }
+      rmdir $tmp_dir;
+      if (-d $tmp_dir) {
+         local $@; local $SIG{__DIE__};
+         eval 'require File::Path; File::Path::rmtree($tmp_dir)';
+      }
+   }
+
+   $tmp_dir = undef;
+}
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Call stop_and_exit when exiting the script.
+## Stops execution, removes temp directory and exits cleanly.
+##
+## Provides safe reentrant logic for parent and child processes.
+## The $main_proc_id variable is defined above.
 ##
 ###############################################################################
 
-END {
-   my $_exit_status = $?;
+my %_sig_name_lkup = map { $_ => 1 } qw(
+   CHLD __DIE__ HUP INT PIPE QUIT TERM __WARN__
+);
 
-   MCE::Signal->_shutdown_mce($_exit_status)
-      if (defined $_mce_spawned_ref);
+my $_count = 0;
 
-   MCE::Signal->stop_and_exit($_exit_status)
-      if ($$ == $main_proc_id);
+my $_handler_count = $INC{'threads/shared.pm'}
+   ? threads::shared::share($_count)
+   : \$_count;
+
+sub stop_and_exit {
+   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
+
+   my ($_exit_status, $_is_sig, $_sig_name) = ($?, 0, $_[0] || 0);
+   $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
+
+   if (exists $_sig_name_lkup{$_sig_name}) {
+      $SIG{INT} = $SIG{$_sig_name} = \&_NOOP,
+      $_is_sig  = $MCE::Signal::KILLED = 1;
+      $_exit_status = 255 if ($_sig_name eq '__DIE__');
+      $_exit_status = 0   if ($_sig_name eq 'PIPE');
+   }
+
+   ## Main process.
+   if ($$ == $main_proc_id) {
+
+      if (++${ $_handler_count } == 1) {
+         ## Kill process group if signaled.
+         if ($_is_sig == 1) {
+            ($_sig_name eq 'PIPE')
+               ? CORE::kill('PIPE', $_is_MSWin32 ? -$$ : -getpgrp)
+               : CORE::kill('INT' , $_is_MSWin32 ? -$$ : -getpgrp);
+
+            if ($_sig_name eq 'PIPE') {
+               for my $_i (1..2) { Time::HiRes::sleep(0.015); }
+            } else {
+               for my $_i (1..3) { Time::HiRes::sleep(0.060); }
+            }
+         }
+
+         ## Remove temp directory.
+         _remove_tmpdir() if defined($tmp_dir);
+
+         ## Signal process group to die.
+         if ($_is_sig == 1) {
+            if ($_sig_name eq 'PIPE' && $INC{'MCE/Hobo.pm'}) {
+               CORE::kill('QUIT', $_is_MSWin32 ? -$$ : -getpgrp);
+            }
+            if ($_sig_name eq 'INT' && -t STDIN) { ## no critic
+               print {*STDERR} "\n";
+            }
+            if ($INC{'threads.pm'} && ($] lt '5.012000' || threads->tid())) {
+               ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
+                  ? CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
+                  : CORE::kill('KILL', -$$, $main_proc_id);
+            }
+            else {
+               CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
+            }
+         }
+      }
+   }
+
+   ## Child processes.
+   elsif ($_is_sig) {
+
+      ## Windows support, from nested workers.
+      if ($_is_MSWin32) {
+         _remove_tmpdir() if defined($tmp_dir);
+         CORE::kill('KILL', $main_proc_id, -$$);
+      }
+
+      ## Real child processes.
+      else {
+         CORE::kill($_sig_name, $main_proc_id, -$$);
+         CORE::kill('KILL', $$, -$$);
+      }
+   }
+
+   ## Exit with status.
+   CORE::exit($_exit_status);
 }
 
 ###############################################################################
@@ -172,7 +277,6 @@ END {
 ###############################################################################
 
 sub sys_cmd {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    _croak('MCE::Signal::sys_cmd: no arguments were specified') if (@_ == 0);
@@ -183,229 +287,15 @@ sub sys_cmd {
 
    ## Kill the process group if command caught SIGINT or SIGQUIT.
 
-   kill('INT',  $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+   CORE::kill('INT',  $main_proc_id, $_is_MSWin32 ? -$$ : -getpgrp)
       if $_sig_no == 2;
 
-   kill('QUIT', $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+   CORE::kill('QUIT', $main_proc_id, $_is_MSWin32 ? -$$ : -getpgrp)
       if $_sig_no == 3;
 
    return $_exit_status;
 }
 
-###############################################################################
-## ----------------------------------------------------------------------------
-## Stops execution, removes temp directory and exits cleanly.
-##
-## Provides safe reentrant logic for both parent and child processes.
-## The $main_proc_id variable is defined above.
-##
-###############################################################################
-
-{
-   my $_handler_cnt = 0;
-
-   my %_sig_name_lkup = map { $_ => 1 } qw(
-      __DIE__ __WARN__ HUP INT PIPE QUIT TERM CHLD XCPU XFSZ
-   );
-
-   sub _NOOP { }
-
-   sub stop_and_exit {
-
-      shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-
-      my $_sig_name        = $_[0] || 0;
-      my $_exit_status     = $?;
-      my $_is_sig          = 0;
-      my $_is_main_thread;
-
-      if (exists $_sig_name_lkup{$_sig_name}) {
-         $_mce_spawned_ref = undef;
-         $SIG{$_sig_name}  = \&_NOOP;
-         $_exit_status     = $_is_sig = 1;
-      } else {
-         $_exit_status = $_sig_name if ($_sig_name ne '0');
-      }
-
-      if ($INC{'threads.pm'}) {
-         $_is_main_thread  = ($$ == $main_proc_id && threads->tid() == 0);
-      } else {
-         $_is_main_thread  = ($$ == $main_proc_id);
-      }
-
-      $SIG{INT} = \&_NOOP if ($_sig_name ne 'INT');
-      $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
-
-      ## ----------------------------------------------------------------------
-
-      ## For the main thread / manager process.
-      if ($_is_main_thread) {
-
-         if (++$_handler_cnt == 1 && ! -e "$tmp_dir/stopped") {
-            open my $_FH, '>', "$tmp_dir/stopped"; close $_FH;
-
-            local $\ = undef;
-
-            ## Display message and kill process group if signaled.
-            if ($_is_sig == 1) {
-               my $_err_msg = undef;
-
-               if ($_sig_name eq 'XCPU') {
-                  $_err_msg = 'exceeded CPU time limit, exiting';
-               }
-               elsif ($_sig_name eq 'XFSZ') {
-                  $_err_msg = 'exceeded file size limit, exiting';
-               }
-               elsif ($_sig_name eq 'INT' && -f "$tmp_dir/died") {
-                  $_err_msg = 'caught signal (__DIE__), exiting';
-               }
-               elsif ($_sig_name eq '__DIE__') {
-                  $_err_msg = 'caught signal (__DIE__), exiting';
-               }
-               elsif ($_sig_name ne 'PIPE') {
-                  $_err_msg = "caught signal ($_sig_name), exiting";
-               }
-
-               ## Display error message.
-               if ($_err_msg && $_no_sigmsg == 0) {
-                  print {*STDERR} "\n## $prog_name: $_err_msg\n";
-               }
-
-               open my $_FH, '>', "$tmp_dir/killed"; close $_FH;
-
-               ## Signal process group to terminate.
-               kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
-
-               ## Pause a bit.
-               if ($_sig_name ne 'PIPE') {
-                  sleep 0.065 for (1..3);
-               } else {
-                  sleep 0.015 for (1..2);
-               }
-            }
-
-            ## Remove temp directory.
-            if (defined $tmp_dir && $tmp_dir ne '' && -d $tmp_dir) {
-
-               if (defined $_mce_sess_dir_ref) {
-                  for my $_sess_dir (keys %{ $_mce_sess_dir_ref }) {
-                     File::Path::rmtree($_sess_dir);
-                     delete $_mce_sess_dir_ref->{$_sess_dir};
-                  }
-               }
-
-               if ($_keep_tmp_dir == 1) {
-                  print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
-               }
-               else {
-                  if ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
-                     File::Path::rmtree($tmp_dir);
-                  }
-               }
-
-               $tmp_dir = undef;
-            }
-
-            ## Signal process group to die.
-            if ($_is_sig == 1) {
-               if ($_sig_name ne 'PIPE' && $_no_sigmsg == 0) {
-                  print {*STDERR} "\n";
-               }
-               if ($_no_kill9 == 1 || $_sig_name eq 'PIPE') {
-                  kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
-               } else {
-                  kill('KILL', -$$, $main_proc_id);
-               }
-            }
-         }
-      }
-
-      ## ----------------------------------------------------------------------
-
-      ## For child processes / threads.
-      if (!$_is_main_thread && $_is_sig == 1 && -d $tmp_dir) {
-
-         ## Signal process group to terminate.
-         if (++$_handler_cnt == 1) {
-
-            ## Obtain lock.
-            open my $CHILD_LOCK, '+>>', "$tmp_dir/child.lock";
-            flock $CHILD_LOCK, LOCK_EX;
-
-            ## Notify the main process that I've died.
-            if ($_sig_name eq '__DIE__' && ! -f "$tmp_dir/died") {
-               local $@; eval {
-                  open my $_FH, '>', "$tmp_dir/died"; close $_FH;
-               };
-            }
-
-            ## Signal process group to terminate.
-            if (! -f "$tmp_dir/killed" && ! -f "$tmp_dir/stopped") {
-               local $@; eval {
-                  open my $_FH, '>', "$tmp_dir/killed"; close $_FH;
-               };
-               if ($_sig_name eq 'PIPE') {
-                  kill('PIPE', $main_proc_id, -$$);
-               } else {
-                  kill('INT', $main_proc_id, -$$);
-               }
-            }
-
-            ## Release lock.
-            flock $CHILD_LOCK, LOCK_UN;
-            close $CHILD_LOCK;
-         }
-      }
-
-      ## ----------------------------------------------------------------------
-
-      ## Exit thread/process with status.
-      if ($_is_sig == 1 && $_no_kill9 == 0) {
-         sleep 0.065 for (1..6);
-      }
-
-      if ($INC{'threads.pm'} && threads->can('exit')) {
-         threads->exit($_exit_status);
-      }
-
-      CORE::exit($_exit_status);
-   }
-}
-
-###############################################################################
-## ----------------------------------------------------------------------------
-## Shutdown MCEs that were previously initiated by this process ID and are
-## still running.
-##
-###############################################################################
-
-sub _shutdown_mce {
-
-   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-   my $_exit_status = $_[0] || $?;
-
-   if (defined $_mce_spawned_ref) {
-      my $_tid = ($INC{'threads.pm'}) ? threads->tid() : '';
-         $_tid = '' unless defined $_tid;
-
-      for my $_mce_sid (keys %{ $_mce_spawned_ref }) {
-         if ($_mce_spawned_ref->{$_mce_sid}->wid()) {
-
-            $_mce_spawned_ref->{$_mce_sid}->exit($_exit_status)
-               if ($_mce_spawned_ref->{$_mce_sid}->pid() == $$);
-         }
-         else {
-            $_mce_spawned_ref->{$_mce_sid}->shutdown()
-               if ($_mce_sid =~ /\A$$\.$_tid\./);
-         }
-
-         delete $_mce_spawned_ref->{$_mce_sid};
-      }
-   }
-
-   return;
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Signal handlers for __DIE__ & __WARN__ utilized by MCE.
@@ -413,7 +303,6 @@ sub _shutdown_mce {
 ###############################################################################
 
 sub _die_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    if (!defined $^S || $^S) {
@@ -422,7 +311,6 @@ sub _die_handler {
       ) {
          # thread env or running inside IPerl, check stack trace
          my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
-
          if ( $_t =~ /^(?:[^\n]+\n){1,7}\teval / ||
               $_t =~ /\n\teval [^\n]+\n\t(?:eval|Try)/ )
          {
@@ -450,17 +338,10 @@ sub _die_handler {
       }
    }
 
-   MCE::Signal->stop_and_exit('__DIE__');
-
-   if ($INC{'threads.pm'} && threads->can('exit')) {
-      threads->exit(255);
-   }
-
-   CORE::exit(255);
+   MCE::Signal::stop_and_exit('__DIE__');
 }
 
 sub _warn_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    ## Ignore thread warnings during exiting.
@@ -492,6 +373,46 @@ sub _warn_handler {
 
 1;
 
+###############################################################################
+## ----------------------------------------------------------------------------
+## TIE scalar package for making $MCE::Signal::tmp_dir on demand.
+##
+###############################################################################
+
+package MCE::Signal::_tmpdir;
+
+sub TIESCALAR {
+   my $_class = shift;
+   bless \do{ my $o = defined $_[0] ? shift : undef }, $_class;
+}
+
+sub STORE {
+   ${ $_[0] } = $_[1];
+
+   $_safe_clean = 0 if ( length $_[1] < 9 );
+   $_safe_clean = 0 if ( $ENV{'TEMP'} && $ENV{'TEMP'} eq $_[1] );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:etc|bin|lib|sbin)} );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:temp|tmp)[\\/]?$}i );
+
+   $_[1];
+}
+
+sub FETCH {
+   if (!defined ${ $_[0] }) {
+      my $_caller = caller();
+      if ($_caller ne 'MCE' && $_caller ne 'MCE::Signal') {
+         if ($INC{'MCE.pm'} && MCE->wid() > 0) {
+            ${ $_[0] } = MCE->tmp_dir();
+         } else {
+            ${ $_[0] } = MCE::Signal::_make_tmpdir();
+         }
+      }
+   }
+   ${ $_[0] };
+}
+
+1;
+
 __END__
 
 ###############################################################################
@@ -506,18 +427,32 @@ MCE::Signal - Temporary directory creation/cleanup and signal handling
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.608
+This document describes MCE::Signal version 1.833
 
 =head1 SYNOPSIS
 
- use MCE::Signal qw( [-keep_tmp_dir] [-use_dev_shm] );
+ ## Creates tmp_dir under $ENV{TEMP} if defined, otherwise /tmp.
 
- use MCE;   ## MCE loads MCE::Signal when not present.
-            ## Include MCE::Signal first for options to take effect.
+ use MCE::Signal;
+
+ ## Attempts to create tmp_dir under /dev/shm if writable.
+
+ use MCE::Signal qw( -use_dev_shm );
+
+ ## Keeps tmp_dir after the script terminates.
+
+ use MCE::Signal qw( -keep_tmp_dir );
+ use MCE::Signal qw( -use_dev_shm -keep_tmp_dir );
+
+ ## MCE loads MCE::Signal by default when not present.
+ ## Therefore, load MCE::Signal first for options to take effect.
+
+ use MCE::Signal qw( -keep_tmp_dir -use_dev_shm );
+ use MCE;
 
 =head1 DESCRIPTION
 
-This package configures $SIG{HUP,INT,PIPE,QUIT,TERM,XCPU,XFSZ} to point to
+This package configures $SIG{ HUP, INT, PIPE, QUIT, and TERM } to point to
 stop_and_exit and creates a temporary directory. The main process and workers
 receiving said signals call stop_and_exit, which signals all workers to
 terminate, removes the temporary directory unless -keep_tmp_dir is specified,
@@ -525,9 +460,7 @@ and terminates itself.
 
 The location of the temp directory resides under $ENV{TEMP} if defined,
 otherwise /dev/shm if writeable and -use_dev_shm is specified, or /tmp.
-
-The temp dir resides under $ENV{TEMP}/mce/ for native Perl on Microsoft
-Windows.
+On Windows, the temp directory is made under $ENV{TEMP}/Perl-MCE/.
 
 As of MCE 1.405, MCE::Signal no longer calls setpgrp by default. Pass the
 -setpgrp option to MCE::Signal to call setpgrp.
@@ -538,18 +471,15 @@ As of MCE 1.405, MCE::Signal no longer calls setpgrp by default. Pass the
  use MCE::Signal qw(-setpgrp);   ## Not necessary for MCE 1.512 and above
  use MCE;
 
-The following are available arguments and their meanings.
+The following are available options and their meanings.
 
  -keep_tmp_dir     - The temporary directory is not removed during exiting
                      A message is displayed with the location afterwards
 
  -use_dev_shm      - Create the temporary directory under /dev/shm
-
- -no_sigmsg        - Do not display a message when receiving a signal
  -no_kill9         - Do not kill -9 after receiving a signal to terminate
 
  -setpgrp          - Calls setpgrp to set the process group for the process
-
                      This option ensures all workers terminate when reading
                      STDIN for MCE releases 1.511 and below.
 
@@ -561,6 +491,9 @@ The following are available arguments and their meanings.
 
 Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
 
+ use MCE::Signal qw( $tmp_dir stop_and_exit sys_cmd );
+ use MCE::Signal qw( :all );
+
  $tmp_dir          - Path to the temporary directory.
  stop_and_exit     - Described below
  sys_cmd           - Described below
@@ -568,14 +501,14 @@ Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
 =head2 stop_and_exit ( [ $exit_status | $signal ] )
 
 Stops execution, removes temp directory, and exits the entire application.
-Pass 'TERM' to terminate a spawned or running MCE state.
+Pass 'INT' to terminate a spawned or running MCE session.
 
  MCE::Signal::stop_and_exit(1);
- MCE::Signal::stop_and_exit('TERM');
+ MCE::Signal::stop_and_exit('INT');
 
 =head2 sys_cmd ( $command )
 
-The system function in Perl ignores SIGNINT and SIGQUIT. These 2 signals are
+The system function in Perl ignores SIGINT and SIGQUIT. These 2 signals are
 sent to the command being executed via system() but not back to the underlying
 Perl script. For this reason, sys_cmd was added to MCE::Signal.
 
@@ -587,22 +520,9 @@ Perl script. For this reason, sys_cmd was added to MCE::Signal.
 
  my $exit_status = sys_cmd($command);
 
-=head1 EXAMPLES
-
- ## Creates tmp_dir under $ENV{TEMP} if defined, otherwise /tmp
- use MCE::Signal;
- use MCE::Signal qw( :all );
-
- ## Attempt to create tmp_dir under /dev/shm if writable
- use MCE::Signal qw( -use_dev_shm );
-
- ## Keep tmp_dir after script terminates
- use MCE::Signal qw( -keep_tmp_dir );
- use MCE::Signal qw( -use_dev_shm -keep_tmp_dir );
-
 =head1 INDEX
 
-L<MCE|MCE>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
