@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Util - Utility functions for Many-Core Engine.
+## Utility functions.
 ##
 ###############################################################################
 
@@ -9,15 +9,24 @@ package MCE::Util;
 use strict;
 use warnings;
 
+no warnings qw( threads recursion uninitialized numeric );
+
+our $VERSION = '1.833';
+
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
-use Socket qw( PF_UNIX PF_UNSPEC SOCK_STREAM );
+use Socket qw( PF_UNIX PF_UNSPEC SOCK_STREAM SOL_SOCKET SO_SNDBUF SO_RCVBUF );
+use Time::HiRes qw( sleep time );
 use base qw( Exporter );
 use bytes;
 
-our $VERSION = '1.608';
+my ($_is_winenv, $_zero_bytes);
+
+BEGIN {
+   $_is_winenv  = ( $^O =~ /mswin|mingw|msys|cygwin/i ) ? 1 : 0;
+   $_zero_bytes = "\x00\x00\x00\x00";
+}
 
-my  $_is_winenv = ($^O eq 'cygwin' || $^O eq 'MSWin32') ? 1 : 0;
 our $LF = "\012";  Internals::SvREADONLY($LF, 1);
 
 our @EXPORT_OK = qw( $LF get_ncpu );
@@ -49,10 +58,10 @@ sub get_ncpu {
       local $_ = lc $^O;
 
       /linux/ && do {
-         my ($count, $fh);
+         my ( $count, $fh );
          if ( open $fh, '<', '/proc/stat' ) {
-             $count = grep { /^cpu\d/ } <$fh>;
-             close $fh;
+            $count = grep { /^cpu\d/ } <$fh>;
+            close $fh;
          }
          $ncpu = $count if $count;
          last OS_CHECK;
@@ -65,12 +74,19 @@ sub get_ncpu {
       };
 
       /aix/ && do {
-         my @output = `pmcycles -m 2>/dev/null`;
-         if (@output) {
-            $ncpu = scalar @output;
-         } else {
-            @output = `lsdev -Cc processor -S Available 2>/dev/null`;
-            $ncpu = scalar @output if @output;
+         my @output = `lparstat -i 2>/dev/null | grep "^Online Virtual CPUs"`;
+         if ( @output ) {
+            $output[0] =~ /(\d+)\n$/;
+            $ncpu = $1 if $1;
+         }
+         if ( !$ncpu ) {
+            @output = `pmcycles -m 2>/dev/null`;
+            if ( @output ) {
+               $ncpu = scalar @output;
+            } else {
+               @output = `lsdev -Cc processor -S Available 2>/dev/null`;
+               $ncpu = scalar @output if @output;
+            }
          }
          last OS_CHECK;
       };
@@ -81,6 +97,12 @@ sub get_ncpu {
          last OS_CHECK;
       };
 
+      /haiku/ && do {
+         my @output = `sysinfo -cpu 2>/dev/null | grep "^CPU #"`;
+         $ncpu = scalar @output if @output;
+         last OS_CHECK;
+      };
+
       /hp-?ux/ && do {
          my $count = grep { /^processor/ } `ioscan -fkC processor 2>/dev/null`;
          $ncpu = $count if $count;
@@ -105,7 +127,7 @@ sub get_ncpu {
          last OS_CHECK;
       };
 
-      /mswin|mingw|cygwin/ && do {
+      /mswin|mingw|msys|cygwin/ && do {
          if (exists $ENV{NUMBER_OF_PROCESSORS}) {
             $ncpu = $ENV{NUMBER_OF_PROCESSORS};
          }
@@ -122,30 +144,59 @@ sub get_ncpu {
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Private methods.
+## Private methods for pipes and sockets.
 ##
 ###############################################################################
 
-sub _destroy_sockets {
+sub _destroy_pipes {
 
    my ($_obj, @_params) = @_;
 
-   local ($!, $?);
+   local ($!,$?); local $SIG{__DIE__};
 
    for my $_p (@_params) {
-      if (defined $_obj->{$_p}) {
-         if (ref $_obj->{$_p} eq 'ARRAY') {
-            for my $_i (0 .. @{ $_obj->{$_p} } - 1) {
-               ## an empty socket may not close immediately in Windows/Cygwin
-               syswrite $_obj->{$_p}->[$_i], '0' if ($_is_winenv); # hack
-               CORE::shutdown $_obj->{$_p}->[$_i], 2;
-               close $_obj->{$_p}->[$_i];
-               undef $_obj->{$_p}->[$_i];
-            }
+      next unless (defined $_obj->{$_p});
+
+      if (ref $_obj->{$_p} eq 'ARRAY') {
+         for my $_i (0 .. @{ $_obj->{$_p} } - 1) {
+            next unless (defined $_obj->{$_p}[$_i]);
+            close $_obj->{$_p}[$_i] if (fileno $_obj->{$_p}[$_i]);
+            undef $_obj->{$_p}[$_i];
          }
-         else {
-            syswrite $_obj->{$_p}, '0' if ($_is_winenv); # ditto
-            CORE::shutdown $_obj->{$_p}, 2;
+      }
+      else {
+         close $_obj->{$_p} if (fileno $_obj->{$_p});
+         undef $_obj->{$_p};
+      }
+   }
+
+   return;
+}
+
+sub _destroy_socks {
+
+   my ($_obj, @_params) = @_;
+
+   local ($!,$?,$@); local $SIG{__DIE__};
+
+   for my $_p (@_params) {
+      next unless (defined $_obj->{$_p});
+
+      if (ref $_obj->{$_p} eq 'ARRAY') {
+         for my $_i (0 .. @{ $_obj->{$_p} } - 1) {
+            next unless (defined $_obj->{$_p}[$_i]);
+            if (fileno $_obj->{$_p}[$_i]) {
+               syswrite($_obj->{$_p}[$_i], '0') if $_is_winenv;
+               eval q{ CORE::shutdown($_obj->{$_p}[$_i], 2) };
+               close $_obj->{$_p}[$_i];
+            }
+            undef $_obj->{$_p}[$_i];
+         }
+      }
+      else {
+         if (fileno $_obj->{$_p}) {
+            syswrite($_obj->{$_p}, '0') if $_is_winenv;
+            eval q{ CORE::shutdown($_obj->{$_p}, 2) };
             close $_obj->{$_p};
          }
          undef $_obj->{$_p};
@@ -155,40 +206,107 @@ sub _destroy_sockets {
    return;
 }
 
-sub _make_socket_pair {
+sub _pipe_pair {
 
    my ($_obj, $_r_sock, $_w_sock, $_i) = @_;
 
-   local $!; my $_old_hndl;
+   local $!;
 
    if (defined $_i) {
-      socketpair( $_obj->{$_r_sock}->[$_i], $_obj->{$_w_sock}->[$_i],
+      # remove tainted'ness
+      ($_i) = $_i =~ /(.*)/;
+
+      pipe($_obj->{$_r_sock}[$_i], $_obj->{$_w_sock}[$_i])
+         or die "pipe: $!\n";
+
+      # IO::Handle->autoflush not available in older Perl.
+      select(( select($_obj->{$_w_sock}[$_i]), $| = 1 )[0]);
+   }
+   else {
+      pipe($_obj->{$_r_sock}, $_obj->{$_w_sock})
+         or die "pipe: $!\n";
+
+      select(( select($_obj->{$_w_sock}), $| = 1 )[0]); # Ditto.
+   }
+
+   return;
+}
+
+sub _sock_pair {
+
+   my ($_obj, $_r_sock, $_w_sock, $_i) = @_;
+
+   my $_size = 16384; local $!;
+
+   if (defined $_i) {
+      # remove tainted'ness
+      ($_i) = $_i =~ /(.*)/;
+
+      socketpair( $_obj->{$_r_sock}[$_i], $_obj->{$_w_sock}[$_i],
          PF_UNIX, SOCK_STREAM, PF_UNSPEC ) or die "socketpair: $!\n";
 
-      binmode $_obj->{$_r_sock}->[$_i];
-      binmode $_obj->{$_w_sock}->[$_i];
+      if ($^O ne 'aix' && $^O ne 'linux') {
+         setsockopt($_obj->{$_r_sock}[$_i], SOL_SOCKET, SO_SNDBUF, int $_size);
+         setsockopt($_obj->{$_r_sock}[$_i], SOL_SOCKET, SO_RCVBUF, int $_size);
+         setsockopt($_obj->{$_w_sock}[$_i], SOL_SOCKET, SO_SNDBUF, int $_size);
+         setsockopt($_obj->{$_w_sock}[$_i], SOL_SOCKET, SO_RCVBUF, int $_size);
+      }
 
-      ## Autoflush handles.
-      $_old_hndl = select $_obj->{$_r_sock}->[$_i]; $| = 1;
-                   select $_obj->{$_w_sock}->[$_i]; $| = 1;
+      # IO::Handle->autoflush not available in older Perl.
+      select(( select($_obj->{$_w_sock}[$_i]), $| = 1 )[0]);
+      select(( select($_obj->{$_r_sock}[$_i]), $| = 1 )[0]);
    }
    else {
       socketpair( $_obj->{$_r_sock}, $_obj->{$_w_sock},
          PF_UNIX, SOCK_STREAM, PF_UNSPEC ) or die "socketpair: $!\n";
 
-      binmode $_obj->{$_r_sock};
-      binmode $_obj->{$_w_sock};
+      if ($^O ne 'aix' && $^O ne 'linux') {
+         setsockopt($_obj->{$_r_sock}, SOL_SOCKET, SO_SNDBUF, int $_size);
+         setsockopt($_obj->{$_r_sock}, SOL_SOCKET, SO_RCVBUF, int $_size);
+         setsockopt($_obj->{$_w_sock}, SOL_SOCKET, SO_SNDBUF, int $_size);
+         setsockopt($_obj->{$_w_sock}, SOL_SOCKET, SO_RCVBUF, int $_size);
+      }
 
-      ## Autoflush handles.
-      $_old_hndl = select $_obj->{$_r_sock}; $| = 1;
-                   select $_obj->{$_w_sock}; $| = 1;
+      select(( select($_obj->{$_w_sock}), $| = 1 )[0]); # Ditto.
+      select(( select($_obj->{$_r_sock}), $| = 1 )[0]);
    }
 
-   select $_old_hndl;
-
    return;
 }
 
+sub _sock_ready {
+
+   my ($_socket, $_timeout) = @_;
+
+   my $_val_bytes = "\x00\x00\x00\x00";
+   my $_ptr_bytes = unpack( 'I', pack('P', $_val_bytes) );
+   my ($_count, $_start) = (1, time);
+
+   $_timeout += time if $_timeout;
+
+   while (1) {
+      # MSWin32 FIONREAD
+      ioctl($_socket, 0x4004667f, $_ptr_bytes);
+
+      return '' if $_val_bytes ne $_zero_bytes;
+      return  1 if $_timeout && time > $_timeout;
+
+      if ($_count) {
+          # delay after a while to not consume a CPU core
+          $_count = 0 if ++$_count % 50 == 0 && time - $_start > 0.005;
+          next;
+      }
+
+      sleep 0.030;
+   }
+}
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Private routines for MCE Models: Flow, Grep, Loop, Map, Step, and Stream.
+##
+###############################################################################
+
 sub _parse_max_workers {
 
    my ($_max_workers) = @_;
@@ -240,13 +358,31 @@ sub _parse_chunk_size {
       if ( (defined $_params && ref $_params->{input_data} eq 'CODE') ||
            (defined $_input_data && ref $_input_data eq 'CODE')
       ) {
+         # Iterators may optionally use chunk_size to determine how much
+         # to return per iteration. The default is 1 for MCE Models, same
+         # as for the Core API. The user_func receives an array_ref
+         # regardless if 1 or higher.
+         #
+         # sub make_iter {
+         #    ...
+         #    return sub {
+         #       my ($chunk_size) = @_;
+         #       ...
+         #    };
+         # }
          return 1;
       }
 
-      my $_size = (defined $_input_data && ref $_input_data eq 'ARRAY')
-         ? scalar @{ $_input_data } : $_array_size;
-
       my $_is_file;
+      my $_size = $_array_size;
+
+      if (defined $_input_data) {
+         if (ref $_input_data eq 'ARRAY') {
+            $_size = scalar @{ $_input_data };
+         } elsif (ref $_input_data eq 'HASH') {
+            $_size = scalar keys %{ $_input_data };
+         }
+      }
 
       if (defined $_params && exists $_params->{sequence}) {
          my ($_begin, $_end, $_step);
@@ -257,9 +393,9 @@ sub _parse_chunk_size {
             $_step  = $_params->{sequence}->{step} || 1;
          }
          else {
-            $_begin = $_params->{sequence}->[0];
-            $_end   = $_params->{sequence}->[1];
-            $_step  = $_params->{sequence}->[2] || 1;
+            $_begin = $_params->{sequence}[0];
+            $_end   = $_params->{sequence}[1];
+            $_step  = $_params->{sequence}[2] || 1;
          }
 
          if (!defined $_input_data && !$_array_size) {
@@ -274,14 +410,14 @@ sub _parse_chunk_size {
          } elsif ($_ref eq '') {
             $_size = -s $_params->{_file};
          } else {
-            $_size = 0; $_chunk_size = 245_760;
+            $_size = 0; $_chunk_size = 393_216;  # 384K
          }
 
          $_is_file = 1;
       }
       elsif (defined $_input_data) {
-         if (ref $_input_data eq 'GLOB' || ref($_input_data) =~ /^IO::/) {
-            $_is_file = 1; $_size = 0; $_chunk_size = 245_760;
+         if (ref($_input_data) =~ /^(?:GLOB|FileHandle|IO::)/) {
+            $_is_file = 1; $_size = 0; $_chunk_size = 393_216;  # 384K
          }
          elsif (ref $_input_data eq 'SCALAR') {
             $_is_file = 1; $_size = length ${ $_input_data };
@@ -291,7 +427,7 @@ sub _parse_chunk_size {
       if (defined $_is_file) {
          if ($_size) {
             $_chunk_size = int($_size / $_max_workers / 24 + 0.5);
-            $_chunk_size = 4_194_304 if $_chunk_size > 4_194_304;  ## 4M
+            $_chunk_size = 5_242_880 if $_chunk_size > 5_242_880;  # 5M
             $_chunk_size = 2 if $_chunk_size <= 8192;
          }
       }
@@ -317,11 +453,11 @@ __END__
 
 =head1 NAME
 
-MCE::Util - Utility functions for Many-Core Engine
+MCE::Util - Utility functions
 
 =head1 VERSION
 
-This document describes MCE::Util version 1.608
+This document describes MCE::Util version 1.833
 
 =head1 SYNOPSIS
 
@@ -351,35 +487,35 @@ This is important for apps which are IO-bound.
 
  my $mce = MCE->new(
 
-   max_workers => 'auto',       ##  1 on HW with 1-lcores;  2 on  2-lcores
-   max_workers =>  16,          ## 16 on HW with 4-lcores; 16 on 32-lcores
+ max_workers => 'auto',       ##  1 on HW with 1-lcores;  2 on  2-lcores
+ max_workers =>  16,          ## 16 on HW with 4-lcores; 16 on 32-lcores
 
-   max_workers => 'auto',       ##  4 on HW with 4-lcores;  8 on 16-lcores
-   max_workers => 'auto*1.5',   ##  4 on HW with 4-lcores; 12 on 16-lcores
-   max_workers => 'auto*2.0',   ##  4 on HW with 4-lcores; 16 on 16-lcores
-   max_workers => 'auto/2.0',   ##  2 on HW with 4-lcores;  4 on 16-lcores
-   max_workers => 'auto+3',     ##  4 on HW with 4-lcores; 11 on 16-lcores
-   max_workers => 'auto-1',     ##  3 on HW with 4-lcores;  7 on 16-lcores
+ max_workers => 'auto',       ##  4 on HW with 4-lcores;  8 on 16-lcores
+ max_workers => 'auto*1.5',   ##  4 on HW with 4-lcores; 12 on 16-lcores
+ max_workers => 'auto*2.0',   ##  4 on HW with 4-lcores; 16 on 16-lcores
+ max_workers => 'auto/2.0',   ##  2 on HW with 4-lcores;  4 on 16-lcores
+ max_workers => 'auto+3',     ##  4 on HW with 4-lcores; 11 on 16-lcores
+ max_workers => 'auto-1',     ##  3 on HW with 4-lcores;  7 on 16-lcores
 
-   max_workers => MCE::Util::get_ncpu,   ## run on all lcores
+ max_workers => MCE::Util::get_ncpu,   ## run on all lcores
  );
 
 In summary:
 
  1. Auto has an upper-limit of 8 in MCE 1.521 (# of lcores, 8 maximum)
- 2. Math can be applied with auto (*/+-) to change the upper limit
+ 2. Math may be applied with auto (*/+-) to change the upper limit
  3. The computed value for auto will not exceed the total # of lcores
- 4. One can specify max_workers explicity to a hard value
+ 4. One can specify max_workers explicitly to a hard value
  5. MCE::Util::get_ncpu returns the actual # of lcores
 
-=head1 ACKNOWLEDGEMENTS
+=head1 ACKNOWLEDGMENTS
 
 The portable code for detecting the number of processors was adopted from
-L<Test::Smoke::SysInfo|Test::Smoke::SysInfo>.
+L<Test::Smoke::SysInfo>.
 
 =head1 INDEX
 
-L<MCE|MCE>
+L<MCE|MCE>, L<MCE::Core>
 
 =head1 AUTHOR
 
